# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions

# global variables
protection_template_to_delete = []
protection_schedule_to_delete = []
prot_template_name1 = nimosclientbase.get_unique_string(
    "unittestcase-protecttemplate1")
prot_schedule_name1 = nimosclientbase.get_unique_string(
    "unittestcase-protectschedule1")


'''protection_schedule TestCase tests the protection_schedule
    object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for protection_schedule TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for protection_schedule TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    global prot_template_name1, prot_schedule_name1
    nimosclientbase.log_header(request.function.__name__)

    prot_template_name1 = nimosclientbase.get_unique_string(
        "unittestcase-protecttemplate1")
    prot_schedule_name1 = nimosclientbase.get_unique_string(
        "unittestcase-protectschedule1")
    resp = create_protection_template(prot_template_name1)
    assert resp is not None
    yield setup_teardown_for_each_test
    # teardown operations below
    delete_protection_template()
    nimosclientbase.log_footer(request.function.__name__)


def delete_protection_template():
    for protection_template_id in protection_template_to_delete:
        try:
            resp = nimosclientbase.get_nimos_client(
            ).protection_templates.delete(id=protection_template_id)
            assert resp is not None
            log("Deleted protection template with "
                f"Id '{protection_template_id}'")
        except exceptions.NimOSAPIError as ex:
            log(f"Failed with exception message : {str(ex)}")
            raise ex
    protection_template_to_delete.clear()


def create_protection_template(protection_template_name):
    resp = nimosclientbase.get_nimos_client().protection_templates.create(
        name=protection_template_name, description="created by testcase")
    protection_template_id = resp.attrs.get("id")
    protection_template_to_delete.append(protection_template_id)
    assert resp is not None
    log("Created protection template with name "
        f"'{protection_template_name}' and Id '{protection_template_id}'")
    return resp


def create_protection_schedule(protection_schedule_name, **kwargs):
    resp = nimosclientbase.get_nimos_client().protection_schedules.create(
        name=protection_schedule_name, **kwargs)
    protection_schedule_id = resp.attrs.get("id")
    protection_schedule_to_delete.append(protection_schedule_id)
    assert resp is not None
    log("Created protection schedule with name "
        f"'{protection_schedule_name}' and Id '{protection_schedule_id}'")
    return resp

@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_protection_schedules(setup_teardown_for_each_test):

    try:
        resp = create_protection_schedule(
            prot_schedule_name1,
            description="created by testcase",
            volcoll_or_prottmpl_id=protection_template_to_delete[0],
            volcoll_or_prottmpl_type="protection_template",
            num_retain=1)
        assert resp is not None
        get_resp = nimosclientbase.get_nimos_client(
        ).protection_schedules.get(id=resp.attrs.get("id"))
        assert get_resp is not None
        assert get_resp.attrs.get("id") == resp.attrs.get("id")

        # retreive all
        getall_resp = nimosclientbase.get_nimos_client(
        ).protection_schedules.list()
        assert getall_resp is not None
        # by default 6 protection schedule are on array
        assert getall_resp.__len__() >= 6
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log(f"Failed as expected. template name : {prot_template_name1}")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_check_mandatory_params_protection_schedules(
        setup_teardown_for_each_test):
    # create requires "volcoll_or_prottmpl_id" and "volcoll_or_
    # prottmpl_type" for creating schedule
    try:
        resp = create_protection_schedule(prot_schedule_name1)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_missing_arg" in str(ex):
            log(f"Failed as expected. mandatory params missing")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_protection_schedules(setup_teardown_for_each_test):
    try:
        resp = create_protection_schedule(
            prot_schedule_name1,
            description="created by testcase",
            volcoll_or_prottmpl_id=protection_template_to_delete[0],
            volcoll_or_prottmpl_type="protection_template",
            num_retain=1)
        assert resp is not None
        assert "created by testcase" == resp.attrs.get("description")
        assert prot_schedule_name1 == resp.attrs.get("name")
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log(f"Failed as expected. some params missing")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_update_protection_schedules(setup_teardown_for_each_test):
    try:
        resp = create_protection_schedule(
            prot_schedule_name1,
            description="created by testcase",
            volcoll_or_prottmpl_id=protection_template_to_delete[0],
            volcoll_or_prottmpl_type="protection_template",
            num_retain=1)
        assert resp is not None
        # update few fields and check it works
        update_resp = nimosclientbase.get_nimos_client(
        ).protection_schedules.update(
            id=resp.attrs.get("id"),
            description="modified by testcase",
            period_unit="hours",
            days="monday",
            num_retain=7)
        assert update_resp is not None
        assert "modified by testcase" == update_resp.attrs.get("description")
        assert 7 == update_resp.attrs.get("num_retain")
        assert update_resp.attrs.get("period_unit") == "hours"
        assert update_resp.attrs.get("days") == "monday"
    except exceptions.NimOSAPIError as ex:
        log(f"Failed with exception message : {str(ex)}")
        raise ex



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class ProtectionSchedule(Resource):
    """Manage protection schedules used in protection templates.

    # Parameters
    id                            : Identifier for protection schedule.
    name                          : Name of snapshot schedule to create.
    description                   : Description of the schedule.
    volcoll_or_prottmpl_type      : Type of the protection policy this schedule is attached to. Valid values are protection_template and volume_collection.
    volcoll_or_prottmpl_id        : Identifier of the protection policy (protection_template or volume_collection) in which this protection schedule is attached to.
    period                        : Repeat interval for snapshots with respect to the period_unit.  For example, a value of 2 with the 'period_unit' of 'hours' results in one
                                    snapshot every 2 hours.
    period_unit                   : Time unit over which to take the number of snapshots specified in 'period'. For example, a value of 'days' with a 'period' of '1' results in
                                    one snapshot every day.
    at_time                       : Time of day when snapshot should be taken. In case repeat frequency specifies more than one snapshot in a day then the until_time option
                                    specifies until what time of day to take snapshots.
    until_time                    : Time of day to stop taking snapshots. Applicable only when repeat frequency specifies more than one snapshot in a day.
    days                          : Specifies which days snapshots should be taken.
    num_retain                    : Number of snapshots to retain. If replication is enabled on this schedule the array will always retain the latest replicated snapshot, which
                                    may exceed the specified retention value. This is necessary to ensure efficient replication performance.
    downstream_partner            : Specifies the partner name if snapshots created by this schedule should be replicated.
    downstream_partner_name       : Specifies the partner name if snapshots created by this schedule should be replicated.
    downstream_partner_id         : Specifies the partner ID if snapshots created by this schedule should be replicated. In an update operation, if snapshots should be replicated,
                                    set this attribute to the ID of the replication partner. If snapshots should not be replicated, set this attribute to the empty string.
    upstream_partner_name         : Specifies the partner name from which snapshots created by this schedule are replicated.
    upstream_partner_id           : Specifies the partner ID from which snapshots created by this schedule are replicated.
    replicate_every               : Specifies which snapshots should be replicated. If snapshots are replicated and this option is not specified, every snapshot is replicated.
    num_retain_replica            : Number of snapshots to retain on the replica.
    repl_alert_thres              : Replication alert threshold in seconds. If the replication of a snapshot takes more than this amount of time to complete an alert will be
                                    generated. Enter 0 to disable this alert.
    snap_verify                   : Run verification tool on snapshot created by this schedule. This option can only be used with snapshot schedules of a protection template that
                                    has application synchronization. The tool used to verify snapshot depends on the type of application. For example, if application
                                    synchronization is VSS and the application ID is Exchange, eseutil tool is run on the snapshots. If verification fails, the logs are not
                                    truncated.
    skip_db_consistency_check     : Skip consistency check for database files on snapshots created by this schedule. This option only applies to snapshot schedules of a protection
                                    template with application synchronization set to VSS, application ID set to MS Exchange 2010 or later w/DAG, this schedule's snap_verify option
                                    set to yes, and its disable_appsync option set to false. Skipping consistency checks is only recommended if each database in a DAG has multiple
                                    copies.
    disable_appsync               : Disables application synchronized snapshots and creates crash consistent snapshots instead.
    schedule_type                 : Normal schedules have internal timers which drive snapshot creation. An externally driven schedule has no internal timers. All snapshot
                                    activity is driven by an external trigger. In other words, these schedules are used only for externally driven manual snapshots.
    active                        : A schedule is active only if it is owned by the same owner as the volume collection. Only active schedules of a volume collection participate
                                    in the creation of snapshots and replication.
    creation_time                 : Time when this protection schedule was created.
    last_modified                 : Time when this protection schedule was last modified.
    last_mod_sched_time           : Time when the timing of the protection schedule was last modified.
    last_replicated_snapcoll_name : Specifies the name of last replicated snapshot collection.
    last_replicated_snapcoll_id   : Specifies the snapshot collection ID of last replicated snapshot collection.
    last_replicated_at_time       : Time when last snapshot collection was replicated.
    last_snap_time                : Time when last snapshot was taken.
    next_snap_time                : Time when next snapshot will be taken.
    next_repl_snap_time           : Time when next snapshot will be replicated.
    snap_counter                  : This is only used by custom read handler for internal calculations.
    sched_owner_id                : Identifier of the group that owns this schedule.
    sched_owner_name              : Name of the group that owns this schedule.
    last_config_change_time       : The last timing configutation changed.
    vol_status_list               : The list of the replication status of volumes undergoing replication.
    sync_repl_vol_status_list     : A list of the replication status of volumes undergoing synchronous replication.
    use_downstream_for_DR         : Break synchronous replication for the specified volume collection and present downstream volumes to host(s). Downstream volumes in the volume
                                    collection will be set to online and presented to the host(s) using new serial and LUN numbers. No changes will be made to the upstream
                                    volumes, their serial and LUN numbers, and their online state. The existing ACLs on the upstream volumes will be copied to the downstream
                                    volumes. Use this in conjunction with an empty downstream_partner_id. This unconfigures synchronous replication when the partner is removed
                                    from the last replicating schedule in the specified volume collection and presents the downstream volumes to host(s). Host(s) will need to be
                                    configured to access the new volumes with the newly assigned serial and LUN numbers. Use this option to expose downstream volumes in a
                                    synchronously replicated volume collection to host(s) only when the upstream partner is confirmed to be down and there is no communication
                                    between partners. Do not execute this operation if a previous Group Management Service takeover has been performed on a different array. Do not
                                    perform a subsequent Group Management Service takeover on a different array as it will lead to irreconcilable conflicts. This limitation is
                                    cleared once the Group management service backup array has successfully synchronized after reconnection.
    """


class ProtectionScheduleList(Collection):
    resource = ProtectionSchedule
    resource_type = "protection_schedules"



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class User(Resource):
    """Represents users configured to manage the system.

    # Parameters
    id                 : Identifier for the user.
    name               : Name of the user.
    search_name        : Name of the user used for object search.
    description        : Description of the user.
    role_id            : Identifier for the user's role.
    role               : Role of the user.
    password           : User's login password.
    auth_password      : Authorization password for changing password.
    inactivity_timeout : The amount of time that the user session is inactive before timing out. A value of 0 indicates that the timeout is taken from the group setting.
    creation_time      : Time when this user was created.
    last_modified      : Time when this user was last modified.
    full_name          : Fully qualified name of the user.
    email_addr         : Email address of the user.
    disabled           : User is currently disabled.
    auth_lock          : User was locked due to failed logins.
    last_login         : Last login time.
    last_logout        : Last logout time.
    logged_in          : User is currently logged in.
    """
    def unlock(self, **kwargs):
        """Unlocks user account locked due to failed logins.

        # Parameters
        id : ID for the user.
        """

        return self._collection.unlock(
            self.id,
            **kwargs
        )


class UserList(Collection):
    resource = User
    resource_type = "users"

    def unlock(self, id, **kwargs):
        """Unlocks user account locked due to failed logins.

        # Parameters
        id : ID for the user.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'unlock',
            id=id,
            **kwargs
        )



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class ProtectionTemplate(Resource):
    """Manage protection templates. Protection templates are sets of snapshot schedules, replication schedules, and retention limits that can be used to prefill the protection
    information when creating new volume collections. A volume collection, once created, is not affected by edits to the protection template that was used to create it. All the
    volumes assigned to a volume collection use the same settings. You cannot edit or delete the predefined protection templates provided by storage array, but you can create
    custom protection templates as needed.

    # Parameters
    id               : Identifier for protection template.
    name             : User provided identifier.
    full_name        : Fully qualified name of protection template.
    search_name      : Name of protection template used for object search.
    description      : Text description of protection template.
    repl_priority    : Replication priority for the protection template with the following choices: {normal | high}.
    app_sync         : Application synchronization ({none|vss|vmware|generic}).
    app_server       : Application server hostname.
    app_id           : Application ID running on the server. Application ID can only be specified if application synchronization is VSS.
    app_cluster_name : If the application is running within a Windows cluster environment then this is the cluster name.
    app_service_name : If the application is running within a Windows cluster environment then this is the instance name of the service running within the cluster environment.
    vcenter_hostname : VMware vCenter hostname. Custom port number can be specified with vCenter hostname using :.
    vcenter_username : VMware vCenter username.
    vcenter_password : VMware vCenter password.
    agent_hostname   : Generic Backup agent hostname. Custom port number can be specified with agent hostname using \\":\\".
    agent_username   : Generic Backup agent username.
    agent_password   : Generic Backup agent password.
    creation_time    : Time when this protection template was created.
    last_modified    : Time when this protection template was last modified.
    schedule_list    : List of schedules for this protection policy.
    """


class ProtectionTemplateList(Collection):
    resource = ProtectionTemplate
    resource_type = "protection_templates"



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class ChapUser(Resource):
    """Manage Challenge-Response Handshake Authentication Protocol (CHAP) user accounts. CHAP users are one method of access control for iSCSI initiators. Each CHAP user has a CHAP
    password, sometimes called a CHAP secret. The CHAP passwords must match on the array and on the iSCSI initiator in order for the array to authenicate the initiator and allow
    it access. The CHAP user information must exist on both the array and the iSCSI initiator. Target authentication gives security only for the specific iSCSI target.

    # Parameters
    id             : Identifier for the CHAP user.
    name           : Name of CHAP user.
    full_name      : CHAP user's fully qualified name.
    search_name    : CHAP user name used for object search.
    description    : Text description of CHAP user.
    password       : CHAP secret.The CHAP secret should be between 12-16 characters and cannot contain spaces or most punctuation.
    initiator_iqns : List of iSCSI initiators. To be configured with this CHAP user for iSCSI Group Target CHAP authentication. This attribute cannot be modified at the same time
                     with other attributes. If any specified initiator is already associated with another CHAP user, it will be replaced by this CHAP user for future CHAP
                     authentication.
    creation_time  : Time when this CHAP user was created.
    last_modified  : Time when this CHAP user was last modified.
    vol_list       : List of volumes associated with this CHAP user.
    vol_count      : Count of volumes associated with this CHAP user.
    """


class ChapUserList(Collection):
    resource = ChapUser
    resource_type = "chap_users"



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class SpaceDomain(Resource):
    """A space domain is created for each application category and block size for each each pool.

    # Parameters
    id                       : Identifier for the space domain.
    pool_id                  : Identifier associated with the pool in the storage pool table.
    pool_name                : Name of the pool containing the space domain.
    app_category_id          : Identifier of the application category associated with the space domain.
    app_category_name        : Name of the application category associated with the space domain.
    perf_policy_names        : Name of the performance policies associated with the space domain.
    sample_rate              : Sample rate value.
    volume_count             : Number of volumes belonging to the space domain.
    deduped_volume_count     : Number of deduplicated volumes belonging to the space domain.
    volumes                  : Volumes belonging to the space domain.
    block_size               : Block size in bytes of volumes belonging to the space domain.
    deduped                  : Volumes in space domain are deduplicated by default.
    encrypted                : Volumes in space domain are encrypted.
    usage                    : Physical space usage of volumes in the space domain.
    vol_logical_usage        : Logical usage of volumes in the space domain.
    snap_logical_usage       : Logical usage of snapshots in the space domain.
    vol_mapped_usage         : Mapped usage of volumes in the space domain, useful for computing clone savings.
    logical_dedupe_usage     : Logical space usage of volumes when deduped.
    physical_dedupe_usage    : Physical space usage of volumes including snapshots when deduped.
    savings_compression      : Space usage savings in the space domain due to compression.
    savings_dedupe           : Space usage savings in the space domain due to deduplication.
    savings_clone            : Space usage savings in the space domain due to cloning of volumes.
    compressed_usage_bytes   : Compressed usage of volumes and snapshots in the space domain.
    uncompressed_usage_bytes : Uncompressed usage of volumes and snapshots in the space domain.
    compression_ratio        : Compression savings for the space domain expressed as ratio.
    dedupe_ratio             : Deduplication savings for the space domain expressed as ratio.
    clone_ratio              : Clone savings for the space domain expressed as ratio.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class SpaceDomainList(Collection):
    resource = SpaceDomain
    resource_type = "space_domains"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class VolumeCollection(Resource):
    """Manage volume collections. Volume collections are logical groups of volumes that share protection characteristics such as snapshot and replication schedules. Volume
    collections can be created from scratch or based on predefined protection templates.

    # Parameters
    id                            : Identifier for volume collection.
    prottmpl_id                   : Identifier of the protection template whose attributes will be used to create this volume collection. This attribute is only used for input
                                    when creating a volume collection and is not outputed.
    name                          : Name of volume collection.
    full_name                     : Fully qualified name of volume collection.
    search_name                   : Name of volume collection used for object search.
    description                   : Text description of volume collection.
    repl_priority                 : Replication priority for the volume collection with the following choices: {normal | high}.
    pol_owner_name                : Owner group.
    replication_type              : Type of replication configured for the volume collection.
    synchronous_replication_type  : Type of synchronous replication configured for the volume collection.
    synchronous_replication_state : State of synchronous replication on the volume collection.
    app_sync                      : Application Synchronization.
    app_server                    : Application server hostname.
    app_id                        : Application ID running on the server. Application ID can only be specified if application synchronization is \\"vss\\".
    app_cluster_name              : If the application is running within a Windows cluster environment, this is the cluster name.
    app_service_name              : If the application is running within a Windows cluster environment then this is the instance name of the service running within the cluster
                                    environment.
    vcenter_hostname              : VMware vCenter hostname. Custom port number can be specified with vCenter hostname using \\":\\".
    vcenter_username              : Application VMware vCenter username.
    vcenter_password              : Application VMware vCenter password.
    agent_hostname                : Generic backup agent hostname. Custom port number can be specified with agent hostname using \\":\\".
    agent_username                : Generic backup agent username.
    agent_password                : Generic backup agent password.
    creation_time                 : Time when this volume collection was created.
    last_modified_time            : Time when this volume collection was last modified.
    volume_list                   : List of volumes associated with the volume collection.
    downstream_volume_list        : List of downstream volumes associated with the volume collection.
    upstream_volume_list          : List of upstream volumes associated with the volume collection.
    volume_count                  : Count of volumes associated with the volume collection.
    cache_pinned_volume_list      : List of cache pinned volumes associated with volume collection.
    last_snapcoll                 : Last snapshot collection on this volume collection.
    snapcoll_count                : Count of snapshot collections associated with volume collection.
    schedule_list                 : List of snapshot schedules associated with volume collection.
    replication_partner           : Replication partner for this volume collection.
    last_replicated_snapcoll      : Last replicated snapshot collection on this volume collection.
    last_replicated_snapcoll_list : List of snapshot collection information for the last replicated snapshot collection per schedule.
    protection_type               : Specifies if volume collection is protected with schedules. If protected, indicated whether replication is setup.
    lag_time                      : Replication lag time for volume collection.
    is_standalone_volcoll         : Indicates whether this is a standalone volume collection.
    total_repl_bytes              : Total size of volumes to be replicated for this volume collection.
    repl_bytes_transferred        : Total size of volumes replicated for this volume collection.
    is_handing_over               : Indicates whether a handover operation is in progress on this volume collection.
    handover_replication_partner  : Replication partner to which ownership is being transferred as part of handover operation.
    metadata                      : Key-value pairs that augment a volume collection's attributes.
    srep_last_sync                : Time when a synchronously replicated volume collection was last synchronized.
    srep_resync_percent           : Percentage of the resync progress for a synchronously replicated volume collection.
    """
    def promote(self, **kwargs):
        """Take ownership of the specified volume collection. The volumes associated with the volume collection will be set to online and be available for reading and writing.
        Replication will be disabled on the affected schedules and must be re-configured if desired. Snapshot retention for the affected schedules will be set to the greater of
        the current local or replica retention values. This operation is not supported for synchronous replication volume collections.

        # Parameters
        id : ID of the promoted volume collection.
        """

        return self._collection.promote(
            self.id,
            **kwargs
        )
    def demote(self, replication_partner_id, **kwargs):
        """Release ownership of the specified volume collection. The volumes associated with the volume collection will set to offline and a snapshot will be created, then full
        control over the volume collection will be transferred to the new owner. This option can be used following a promote to revert the volume collection back to its prior
        configured state. This operation does not alter the configuration on the new owner itself, but does require the new owner to be running in order to obtain its identity
        information. This operation is not supported for synchronous replication volume collections.

        # Parameters
        id                         : ID of the demoted volume collection.
        replication_partner_id     : ID of the new owner. If invoke_on_upstream_partner is provided, utilize the ID of the current owner i.e. upstream replication partner.
        invoke_on_upstream_partner : Invoke demote request on upstream partner. Default: 'false'. This operation is not supported for synchronous replication volume vollections.
        """

        return self._collection.demote(
            self.id,
            replication_partner_id,
            **kwargs
        )
    def handover(self, replication_partner_id, **kwargs):
        """Gracefully transfer ownership of the specified volume collection. This action can be used to pass control of the volume collection to the downstream replication partner.
        Ownership and full control over the volume collection will be given to the downstream replication partner. The volumes associated with the volume collection will be set to
        offline prior to the final snapshot being taken and replicated, thus ensuring full data synchronization as part of the transfer. By default, the new owner will
        automatically begin replicating the volume collection back to this node when the handover completes.

        # Parameters
        id                         : ID of the volume collection be handed over to the downstream replication partner.
        replication_partner_id     : ID of the new owner.
        no_reverse                 : Do not automatically reverse direction of replication. Using this argument will prevent the new owner from automatically replicating the
                                     volume collection to this node when the handover completes. The default behavior is to enable replication back to this node. Default: 'false'.
        invoke_on_upstream_partner : Invoke handover request on upstream partner. Default: 'false'. This operation is not supported for synchronous replication volume vollections.
        override_upstream_down     : Allow the handover request to proceed even if upstream array is down. The default behavior is to return an error when upstream is down. This
                                     option is applicable for synchronous replication only. Default: 'false'.
        """

        return self._collection.handover(
            self.id,
            replication_partner_id,
            **kwargs
        )
    def abort_handover(self, **kwargs):
        """Abort in-progress handover. If for some reason a previously invoked handover request is unable to complete, this action can be used to cancel it. This operation is not
        supported for synchronous replication volume collections.

        # Parameters
        id : ID of the volume collection on which to abort handover.
        """

        return self._collection.abort_handover(
            self.id,
            **kwargs
        )
    def validate(self, **kwargs):
        """Validate a volume collection with either Microsoft VSS or VMware application synchronization.

        # Parameters
        id : ID of the volume collection that is to be validated.
        """

        return self._collection.validate(
            self.id,
            **kwargs
        )


class VolumeCollectionList(Collection):
    resource = VolumeCollection
    resource_type = "volume_collections"

    def promote(self, id, **kwargs):
        """Take ownership of the specified volume collection. The volumes associated with the volume collection will be set to online and be available for reading and writing.
        Replication will be disabled on the affected schedules and must be re-configured if desired. Snapshot retention for the affected schedules will be set to the greater of
        the current local or replica retention values. This operation is not supported for synchronous replication volume collections.

        # Parameters
        id : ID of the promoted volume collection.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'promote',
            id=id,
            **kwargs
        )

    def demote(self, id, replication_partner_id, **kwargs):
        """Release ownership of the specified volume collection. The volumes associated with the volume collection will set to offline and a snapshot will be created, then full
        control over the volume collection will be transferred to the new owner. This option can be used following a promote to revert the volume collection back to its prior
        configured state. This operation does not alter the configuration on the new owner itself, but does require the new owner to be running in order to obtain its identity
        information. This operation is not supported for synchronous replication volume collections.

        # Parameters
        id                         : ID of the demoted volume collection.
        replication_partner_id     : ID of the new owner. If invoke_on_upstream_partner is provided, utilize the ID of the current owner i.e. upstream replication partner.
        invoke_on_upstream_partner : Invoke demote request on upstream partner. Default: 'false'. This operation is not supported for synchronous replication volume vollections.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'demote',
            id=id,
            replication_partner_id=replication_partner_id,
            **kwargs
        )

    def handover(self, id, replication_partner_id, **kwargs):
        """Gracefully transfer ownership of the specified volume collection. This action can be used to pass control of the volume collection to the downstream replication partner.
        Ownership and full control over the volume collection will be given to the downstream replication partner. The volumes associated with the volume collection will be set to
        offline prior to the final snapshot being taken and replicated, thus ensuring full data synchronization as part of the transfer. By default, the new owner will
        automatically begin replicating the volume collection back to this node when the handover completes.

        # Parameters
        id                         : ID of the volume collection be handed over to the downstream replication partner.
        replication_partner_id     : ID of the new owner.
        no_reverse                 : Do not automatically reverse direction of replication. Using this argument will prevent the new owner from automatically replicating the
                                     volume collection to this node when the handover completes. The default behavior is to enable replication back to this node. Default: 'false'.
        invoke_on_upstream_partner : Invoke handover request on upstream partner. Default: 'false'. This operation is not supported for synchronous replication volume vollections.
        override_upstream_down     : Allow the handover request to proceed even if upstream array is down. The default behavior is to return an error when upstream is down. This
                                     option is applicable for synchronous replication only. Default: 'false'.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'handover',
            id=id,
            replication_partner_id=replication_partner_id,
            **kwargs
        )

    def abort_handover(self, id, **kwargs):
        """Abort in-progress handover. If for some reason a previously invoked handover request is unable to complete, this action can be used to cancel it. This operation is not
        supported for synchronous replication volume collections.

        # Parameters
        id : ID of the volume collection on which to abort handover.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'abort_handover',
            id=id,
            **kwargs
        )

    def validate(self, id, **kwargs):
        """Validate a volume collection with either Microsoft VSS or VMware application synchronization.

        # Parameters
        id : ID of the volume collection that is to be validated.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'validate',
            id=id,
            **kwargs
        )



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan
import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions

'''GroupsTestCase tests the subnet object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for Groups TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for Groups TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    nimosclientbase.log_footer(request.function.__name__)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_groups(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().groups.list(
        detail=True, limit=2)
    assert resp is not None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_group_discovered_list(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().groups.get()
        assert resp is not None
        resp = nimosclientbase.get_nimos_client(
        ).groups.get_group_discovered_list(id=resp.attrs.get("id"))
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_array_not_found" in str(ex):
            log("Failed as expected. Array id given is invalid")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_test_alert(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().groups.get()
        assert resp is not None
        test_resp = nimosclientbase.get_nimos_client().groups.test_alert(
            id=resp.attrs.get("id"), level="notice")
        test_resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_array_not_found" in str(ex):
            log("Failed as expected. Array id given is invalid")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_update_group(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().groups.get()
        assert resp is not None
        # save the orig value
        orig_name = resp.attrs.get("name")
        alert_to_email_addrs = resp.attrs.get(
            "alert_to_email_addrs")  # "abc@hpe.com"
        send_alert_to_support = resp.attrs.get("send_alert_to_support")
        isns_enabled = resp.attrs.get("isns_enabled")

        update_resp = nimosclientbase.get_nimos_client().groups.update(
            id=resp.attrs.get("id"),
            name="testname",
            alert_to_email_addrs="alok.ranjxxx@hpe.com",
            send_alert_to_support=False,
            isns_enabled=True)
        update_resp is not None
        # assert the values
        assert update_resp.attrs.get("name") == "testname"
        assert update_resp.attrs.get(
            "alert_to_email_addrs") == "alok.ranjxxx@hpe.com"
        assert update_resp.attrs.get("send_alert_to_support") is False
        assert update_resp.attrs.get("isns_enabled") is True

        # revert to original
        update_resp = nimosclientbase.get_nimos_client().groups.update(
            id=resp.attrs.get("id"),
            name=orig_name,
            alert_to_email_addrs=alert_to_email_addrs,
            send_alert_to_support=send_alert_to_support,
            isns_enabled=isns_enabled)
    except exceptions.NimOSAPIError as ex:
        if "SM_array_not_found" in str(ex):
            log("Failed as expected. Array id given is invalid")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class Controller(Resource):
    """Controller is a redundant collection of hardware capable of running the array software.

    # Parameters
    id                  : Identifier of the controller.
    name                : Name of the controller.
    array_name          : Name of the array containing this controller.
    array_id            : Rest ID of the array containing this controller.
    partial_response_ok : Indicate that it is ok to provide partially available response.
    serial              : Serial number for this controller.
    model               : Model of this controller.
    hostname            : Host name for the controller.
    support_address     : IP address used for support.
    support_netmask     : IP netmask used for support.
    support_nic         : Network card used for support.
    power_status        : Overall power supply status for the controller.
    fan_status          : Overall fan status for the controller.
    temperature_status  : Overall temperature status for the controller.
    power_supplies      : Status for each power supply in the controller.
    fans                : Status for each fan in the controller.
    temperature_sensors : Status for temperature sensor in the controller.
    partition_status    : Status of the system's raid partitions.
    ctrlr_side          : Identifies which controller this is on its array.
    state               : Indicates whether this controller is active or not.
    nvme_cards_enabled  : Indicates if the NVMe accelerator card is enabled.
    nvme_cards          : List of NVMe accelerator cards.
    asup_time           : Time of the last autosupport by the controller.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class ControllerList(Collection):
    resource = Controller
    resource_type = "controllers"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#

import setuptools
import os

# Read the version from nimbleresults.__init__.__version__
base_path = os.path.dirname(__file__)
with open(os.path.join(base_path, 'nimbleclient', '__init__.py')) as fh:
    info = fh.read()
    version = [line for line in info.split('\n') if line.startswith('__version__')][0].split(' = ')[1][1:-1]

with open('README.md', 'r') as fh:
    long_description = fh.read()

with open('requirements.txt') as requirements:
    install_requires = requirements.readlines()

setuptools.setup(
    name='nimble-sdk',
    version=version,
    author='HPE Nimble Storage DCS',
    author_email='nimble-dcs-storage-automation-eng@hpe.com',
    maintainer='Suneethkumar Byadarahalli, Alok Ranjan, George Costea',
    keywords='HPE Nimble Storage REST API SDK',
    description='The HPE Nimble Storage SDK for Python (client library) is a utility that can be leveraged to manage HPE Nimble Storage arrays.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='Apache License, Version 2.0',
    url='https://github.com/hpe-storage/nimble-python-sdk',
    project_urls={
        'Documentation': 'https://hpe-storage.github.io/nimble-python-sdk',
        'Source': 'https://github.com/hpe-storage/nimble-python-sdk',
        'Tracker': 'https://github.com/hpe-storage/nimble-python-sdk/issues',
        'Community': 'https://hpedev.io'
    },
    packages=setuptools.find_packages(),
    install_requires=install_requires,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries',
        'Topic :: Utilities',
    ],
    python_requires='>=3.6',
)



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions, query
from nimbleclient.v1 import query_fields
import time
import threading

'''VolumeTestCase tests the volume object functionality '''

# global variables
vol_name1 = nimosclientbase.get_unique_string("volumetc-vol1")
vol_name2 = nimosclientbase.get_unique_string("volumetc-vol2")
vol_name3 = nimosclientbase.get_unique_string("volumetc-vol3")
vol_to_delete = []
delete_volume_counter = 5  # we will try to deletevolume at max 5 times
volume_lock = threading.RLock()


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for Volume TestCase *****\n")
    cleanup_old_volumes()

    def after_running_all_testcase():
        log("**** Completed Tests for Volume TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


def cleanup_old_volumes():

    # Due to bulk_move operation most of the times when abort is issued.
    # the array ignores the request to delete the volume as the operation
    # is already in progress and hence the volumes are left over. Hence,
    # every time this test is run, we will try to remove the old entries
    log("Cleaning up unwanted volumes if any")
    global delete_volume_counter
    vol_resp = nimosclientbase.get_nimos_client().volumes.list(detail=True)
    volume_lock.acquire()
    for vol_obj in vol_resp:
        while delete_volume_counter != 0:
            try:
                if (str.startswith(vol_obj.attrs.get("name"), "volumetc-vol")
                    or str.startswith(vol_obj.attrs.get("name"),
                                      "clone-volumetc")
                    or str.startswith(vol_obj.attrs.get("name"),
                                      "snapcolltc-vol")):

                    vol_name = vol_obj.attrs.get("name")
                    nimosclientbase.get_nimos_client(
                    ).volumes.offline(vol_obj.attrs.get("id"))
                    nimosclientbase.get_nimos_client(
                    ).volumes.delete(vol_obj.attrs.get("id"))
                    log(f"Deleted volume '{vol_name}'")
                    time.sleep(2)
                break  # break from inner while loop
            except exceptions.NimOSAPIError as ex:
                if ("SM_volmv_vol_einprog" in str(ex)
                        or "SM_vol_connection_count_unavailable" in str(ex)):
                    log("'SM_volmv_vol_einprog' in progress.Trying to delete "
                        f"volume '{vol_name}' again after 3 minutes")
                    time.sleep(180)
                    delete_volume_counter = delete_volume_counter - 1
                elif "SM_vol_has_online_snap" in str(ex):
                    log("Failed as expected with exception : 'SM_vol_has_online_snap'")
                    # offilne the snapshot
                    snapshotlist = vol_obj.attrs.get("online_snaps")
                    for snapid in snapshotlist:
                        nimosclientbase.get_nimos_client(
                        ).snapshots.update(id=snapid["id"], online=False)
                    nimosclientbase.get_nimos_client(
                    ).volumes.delete(vol_obj.attrs.get("id"))
                elif ("SM_vol_assoc_volcoll" in str(ex)):
                    nimosclientbase.get_nimos_client().volumes.update(
                        id=vol_obj.attrs.get("id"), volcoll_id="")  # disassociate
                    nimosclientbase.get_nimos_client(
                    ).volumes.delete(vol_obj.attrs.get("id"))
                else:
                    break  # from while loop
            except Exception as ex:
                log(f"Failed with exception message : {str(ex)}")
                raise ex
    volume_lock.release()


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    global vol_name1, vol_name2, vol_name3
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    # create new volume names for the next testcase
    vol_name1 = nimosclientbase.get_unique_string("volumetc-vol1")
    vol_name2 = nimosclientbase.get_unique_string("volumetc-vol2")
    vol_name3 = nimosclientbase.get_unique_string("volumetc-vol3")
    yield setup_teardown_for_each_test
    # teardown operations below
    delete_volume()
    nimosclientbase.log_footer(request.function.__name__)


def create_volume(vol_name, size=50, read_only="false"):
    volume_lock.acquire()
    try:
        resp = nimosclientbase.get_nimos_client().volumes.create(
            vol_name, size=size, read_only=read_only)
        vol_id = resp.attrs.get("id")
        vol_to_delete.append(vol_id)
        assert resp is not None
        log(f"Created volume with name '{vol_name}' and Id '{vol_id}'")
        volume_lock.release()
        return resp
    except Exception as ex:
        volume_lock.release()
        log(f"Failed with exception message : {str(ex)}")
        raise ex


def delete_volume():
    volume_lock.acquire()
    for vol_id in vol_to_delete:
        try:
            # check
            nimosclientbase.get_nimos_client().volumes.update(
                id=vol_id, volcoll_id="")  # disassociate
            nimosclientbase.get_nimos_client().volumes.offline(vol_id)
            nimosclientbase.get_nimos_client().volumes.delete(vol_id)
            log(f" Deleted volume with id '{vol_id}'")
        except Exception as ex:
            if ("SM_vol_connection_count_unavailable" in str(ex) or
                    "SM_http_service_unavailable" in str(ex)):
                log("Failed to delete volume. Will try next time. "
                    f"Msg: Unable to check for host connectivity to volume")
            elif "SM_volmv_vol_einprog" in str(ex):
                log("Failed to delete volume. Will try next time")
            else:
                volume_lock.release()
                log(f"Failed with exception message : {str(ex)}")
                raise ex
    vol_to_delete.clear()
    volume_lock.release()


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_create_volume(setup_teardown_for_each_test):
    create_volume(vol_name1)
    # check
    vol1 = nimosclientbase.get_nimos_client().volumes.get(id=None,
                                                          name=vol_name1)
    assert vol1 is not None
    assert vol_name1 == vol1.attrs.get("name")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_volume_already_exists(setup_teardown_for_each_test):
    create_volume(vol_name1)
    try:
        # now try creating the same volume again
        create_volume(vol_name1)
    except exceptions.NimOSAPIError as ex:
        # covered SM_eexist and SM_http_conflict
        if 'SM_eexist' in str(ex):
            log(f"Failed As Expected")
    except Exception as exgeneral:
        log(exgeneral)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_volume_bad_params(setup_teardown_for_each_test):
    # , or / is not supported by nimble for creating vol name
    vol_name = vol_name1 + "/,"
    try:
        create_volume(vol_name)
    except exceptions.NimOSAPIError as ex:
        # protocol ex is SM_http_bad_request
        if 'SM_invalid_arg_value' in str(ex):
            log(f"Failed As Expected. Invalid vol_name to create: {vol_name}")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_volume_unexpected_arg(setup_teardown_for_each_test):
    try:
        # "invalidarg" is not a part of volume argument.
        nimosclientbase.get_nimos_client().volumes.create(
            vol_name1, size=50, invalidarg="testinvalidarg")
    except exceptions.NimOSAPIError as ex:
        # protocol ex is SM_http_bad_request
        if 'SM_unexpected_arg' in str(ex):
            log("Failed As Expected. "
                f"Unexpected arg for vol_name : {vol_name1}")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_volume_page_size(setup_teardown_for_each_test):
    resp_vol = []
    # first atleast create few volume
    for i in range(0, 5):
        vol_name = nimosclientbase.get_unique_string("volumetc-vol1-" + str(i))
        resp = create_volume(vol_name)
        resp_vol.append(resp)
    resp = nimosclientbase.get_nimos_client().volumes.list(
        detail=True, limit=2)
    assert resp is not None
    log("Successfully got just 2 items per page")
    assert resp.__len__() == 2


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_invalid_volume_page_size(setup_teardown_for_each_test):
    # first atleast create few volume
    for i in range(0, 6):
        vol_name = nimosclientbase.get_unique_string("volumetc-vol1-" + str(i))
        create_volume(vol_name)
    try:
        nimosclientbase.get_nimos_client().volumes.list(
            detail=True, limit=5000)
    except Exception as ex:
        if"SM_too_large_page_size" in str(ex):
            log("Failed as expected. Invaild limit given")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_volume_readonly(setup_teardown_for_each_test):
    # create a read only volume.But the volume gets created as write only.
    # which makes sense. why would someone create a volume as read only.. but
    # doc says we can..file a bug.
    create_volume(vol_name1, 50, "true")
    # check
    vol1 = nimosclientbase.get_nimos_client().volumes.get(id=None,
                                                          name=vol_name1)
    assert vol1 is not None
    assert vol1.attrs.get("read_only") is False


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_volume_with_few_fields(setup_teardown_for_each_test):
    create_volume(vol_name1)
    resp = nimosclientbase.get_nimos_client().volumes.get()
    assert resp is not None
    log(resp)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_selected_fields_for_all_volumes(setup_teardown_for_each_test):
    # first atleast create few volume
    vol1 = create_volume(vol_name1)
    create_volume(vol_name2)
    resp = nimosclientbase.get_nimos_client().volumes.list(
        detail=True, fields="name,id,size")
    for obj in resp:
        # first match the volume name. there could be that there were
        # volumes on the array than the ones we created also, we will
        # just check the attrs of one volume. if that works fine it will
        # work for other
        if(vol1.attrs.get("name") == obj.attrs.get("name")):
            assert obj.attrs.get("id") == vol1.attrs.get("id")
            assert obj.attrs.get("size") == vol1.attrs.get("size")
            # now check no other extra attributes were returned from server
            # for this volume
            # try fetching some attribute
            assert obj.attrs.get("read_only") is None


# TODO: below function will be implemented when sdk is ready
# for accepting filters
@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_select_fields_for_filtered_volumes(setup_teardown_for_each_test):

    # first atleast create few volume
    resp = create_volume(vol_name1)
    # get by name
    vol1 = nimosclientbase.get_nimos_client().volumes.get(
        id=None, name=vol_name1, fields="name,id,size")
    assert vol1 is not None
    assert resp.attrs.get("id") == vol1.attrs.get("id")
    assert resp.attrs.get("size") == vol1.attrs.get("size")
    assert resp.attrs.get("name") == vol1.attrs.get("name")
    # now check no other extra attributes were returned from server
    # for this volume
    assert vol1.attrs.get("full_name") is None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_non_existent_volumes(setup_teardown_for_each_test):

    resp = nimosclientbase.get_nimos_client().volumes.get(
        name="nonexistentvolume")
    assert resp is None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_sortby_asc_volume_name(setup_teardown_for_each_test):

    vol_name1 = nimosclientbase.get_unique_string("volumetc-vol-vol1-z")
    vol_name2 = nimosclientbase.get_unique_string("volumetc-vol-vol1-a")
    create_volume(vol_name1)
    create_volume(vol_name2)
    resp = nimosclientbase.get_nimos_client().volumes.list(
        detail=False, sortBy="name")  # ascending
    # asert that the resp contains only 2 objects. if more than 2 objects
    # that means the array has volumes which are not created by test.
    # in that case just fail
    if resp.__len__() != 2:
        log("Array has volumes which were not created by unit testcase."
            " make sure array has no volume present before running the test")
    else:
        # after sorting, vol_name2 should be the first object and 2nd object
        # should be vol_name1.
        assert vol_name2 == resp[0].attrs.get("name")
        assert vol_name1 == resp[1].attrs.get("name")


# the belwo test actually also covers the message
# "SM_start_row_beyond_total_rows" and "SM_start_row_beyond_end_rows"
@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_volume_startrow_beyond_endrow_volume(setup_teardown_for_each_test):
    create_volume(vol_name1)
    create_volume(vol_name2)
    # try to read from startrow 7. total only 2 rows will be there and
    # hence the resp should be none
    try:
        resp = nimosclientbase.get_nimos_client().volumes.get(startRow=50)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_start_row_beyond_total_rows" in str(ex):
            pass
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_volume_startrow_equals_endrow_volume(setup_teardown_for_each_test):

    create_volume(vol_name1)
    create_volume(vol_name2)
    try:
        resp = nimosclientbase.get_nimos_client().volumes.get(
            startRow=1, endRow=1)
        assert resp is None
    except exceptions.NimOSAPIError as ex:
        log(f"Failed with exception message : {str(ex)}")
        raise ex

@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_update_volume_size_attribute(setup_teardown_for_each_test):
    resp = create_volume(vol_name1)
    # update the size to 100
    resp = nimosclientbase.get_nimos_client().volumes.update(
        id=resp.attrs.get("id"), size=100)
    assert resp is not None
    # check the size is updated correctly
    vol = nimosclientbase.get_nimos_client().volumes.get(
        id=resp.attrs.get("id"), fields="name,id,size")
    assert resp.attrs.get("size") == vol.attrs.get("size")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_query_volumes_based_on_metadata(
        setup_teardown_for_each_test):
    # first atleast create few volume
    resp_vol1 = create_volume(vol_name1)
    resp_vol2 = create_volume(vol_name2)
    create_volume(vol_name3)
    metadata = {
        "key_-*1": "xyz"
    }
    try:
        client = nimosclientbase.get_nimos_client()
        # update the size to 100 for vol1
        update_resp = nimosclientbase.get_nimos_client().volumes.update(
            id=resp_vol1.attrs.get("id"), metadata=metadata)
        assert update_resp is not None
        # query the volume based on metadata
        query_resp = client.volumes.list(detail=True, filter=query.and_(
            query_fields.VolumeFields.name.contains(resp_vol1.attrs.get("name")),
            query_fields.VolumeFields.metadata("key_-*1") == "xyz"))
        assert query_resp is not None
        # see if the query result has just one volume and match the name
        assert query_resp.__len__() == 1
        assert query_resp[0].attrs.get("name") == resp_vol1.attrs.get("name")
        query_resp_metadata = query_resp[0].attrs.get("metadata")
        assert query_resp_metadata[0]["value"] == metadata['key_-*1']

        # query the volumes without metadata
        query_resp = client.volumes.list(detail=True, filter=query.or_(
            query_fields.VolumeFields.name.contains(resp_vol2.attrs.get("name")),
            query_fields.VolumeFields.metadata("key_-*1") == "xyz"))
        assert query_resp is not None
        # see if the query result has two volume
        assert query_resp.__len__() == 2
    except exceptions.NimOSAPIError as ex:
        log(f"Failed with exception message : {str(ex)}")
        raise ex
    except Exception as ex:
        log(f"Failed with exception message : {str(ex)}")
        raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_update_volume_metadata(setup_teardown_for_each_test):
    resp_vol1 = create_volume(vol_name1)
    create_volume(vol_name2)
    metadata = {
        'key1': 'abcde',
        'key_-*1': "xyz"
    }
    try:
        # update the size to 100
        nimosclientbase.get_nimos_client().volumes.update(
            id=resp_vol1.attrs.get("id"), metadata=metadata)
    except exceptions.NimOSAPIError as ex:
        log(f"Failed with exception message : {str(ex)}")
        raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_update_resize_volume_force(setup_teardown_for_each_test):
    # shrinking an online volume is an operation which will fail
    # unless force=true is specified. This tests the
    # force update object functionality.

    # first atleast create few volume
    resp = create_volume(vol_name1)
    assert resp is not None
    assert resp.attrs.get("size") == 50
    # assert that the volume is online
    assert resp.attrs.get("online") is True
    try:
        # update the size to 100
        resp = nimosclientbase.get_nimos_client().volumes.update(
            id=resp.attrs.get("id"), size=5)
    except exceptions.NimOSAPIError as ex:
        if 'SM_vol_size_decreased' in str(ex):
            assert "the operation will decrease the size of the volume. "
            "use force option to proceed." in str(ex)
    # try with force option
    resp = nimosclientbase.get_nimos_client().volumes.update(
        id=resp.attrs.get("id"), size=5, force=True)
    assert resp.attrs.get("size") == 5


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_clone_volume(setup_teardown_for_each_test):
    # first atleast create few volume
    clone_vol_name = "test.volumeclone.clone"
    resp_vol = create_volume(vol_name1)
    snap_resp = nimosclientbase.get_nimos_client().snapshots.create(
        name="test.volumeclone.snapshot", vol_id=resp_vol.attrs.get("id"))
    # clone from snapshot
    clonevol_resp = nimosclientbase.get_nimos_client().volumes.create(
        name=clone_vol_name,
        base_snap_id=snap_resp.attrs.get("id"),
        clone=True)
    # confirm
    assert clone_vol_name == clonevol_resp.attrs.get("name")
    assert clonevol_resp.attrs.get("size") == resp_vol.attrs.get("size")
    # cleanup
    nimosclientbase.get_nimos_client().volumes.offline(
        clonevol_resp.attrs.get("id"))
    nimosclientbase.get_nimos_client().volumes.delete(
        clonevol_resp.attrs.get("id"))


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_delete_clone_volume(setup_teardown_for_each_test):

    # first create volume
    clone_vol_name = nimosclientbase.get_unique_string("clone-volumetc")
    resp_vol = create_volume(vol_name1)
    vol_id = resp_vol.attrs.get("id")
    # clone a volume
    snap_resp = nimosclientbase.get_nimos_client().snapshots.create(
        name="test.volumeclone.snapshot", vol_id=vol_id)
    clonevol_resp = nimosclientbase.get_nimos_client().volumes.create(
        name=clone_vol_name,
        base_snap_id=snap_resp.attrs.get("id"),
        clone=True)
    # confirm
    assert clone_vol_name == clonevol_resp.attrs.get("name")
    assert clonevol_resp.attrs.get("size") == resp_vol.attrs.get("size")
    # try deleting parent volume . it should fail with
    # exception "SM_vol_has_clone"
    try:
        nimosclientbase.get_nimos_client().volumes.offline(vol_id)
        nimosclientbase.get_nimos_client().volumes.delete(vol_id)
    except exceptions.NimOSAPIError as ex:
        if "SM_vol_has_clone" in str(ex):
            log("Failed as expected. volume has clone")
            # cleanup
            nimosclientbase.get_nimos_client().volumes.offline(
                clonevol_resp.attrs.get("id"))
            nimosclientbase.get_nimos_client().volumes.delete(
                clonevol_resp.attrs.get("id"))
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True or nimosclientbase.get_nimos_client().pools.list().__len__() < 2,
                    reason="skipped this test as SKIPTEST variable is true or its not a multi pool setup")
def test_bulk_move_volume(setup_teardown_for_each_test):
    try:
        orig_vol_pool_name = ""
        # first atleast create few volume
        resp = create_volume(vol_name3, size=5)
        orig_vol_pool_name = resp.attrs.get("pool_name")
        # get the dest pool id.
        pool_resp = nimosclientbase.get_nimos_client().pools.list()
        assert pool_resp is not None
        # make sure we have 2 pools atleaset
        assert pool_resp.__len__() >= 2
        # get the pool id where to move the volume
        for poolobj in pool_resp:
            if poolobj.attrs.get("name") == orig_vol_pool_name:
                continue
            else:
                break
        pool_id = poolobj.attrs.get("id")
        # move the volumes
        moveresp = nimosclientbase.get_nimos_client(1).volumes.bulk_move(
            dest_pool_id=pool_id, vol_ids=vol_to_delete)
        assert moveresp is not None
        # abort the move now
        for vol_id in vol_to_delete:
            abortresp = nimosclientbase.get_nimos_client().volumes.abort_move(
                vol_id)
            assert abortresp is not None
    except exceptions.NimOSAPIError as ex:
        if ("SM_vol_connection_count_unavailable" in str(ex)
                or "SM_vol_usage_unavailable" in str(ex)):
            log("Failed to abort volume move due to service inavailiability")
        elif "SM_invalid_arg_value" in str(ex):
            log("Failed as expected. Invalid pool id provided")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex
    except exceptions.NimOSClientJobTimeoutError as ex:
        log(f"Failed as the job operation has timed out : {str(ex)}")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_move_volume(setup_teardown_for_each_test):

    try:
        create_volume(vol_name3, size=5)
        # deliberately pass wrong pool id. move operation takes lot of time
        # hence we will just call the sdk api to make sure it takes all
        # the arguments
        moveresp = nimosclientbase.get_nimos_client().volumes.move(
            dest_pool_id="kasdkashdqwkdhkas28623612000ef", id=vol_to_delete[0])
        assert moveresp is not None
        # abort the move now
        for vol_id in vol_to_delete:
            abortresp = nimosclientbase.get_nimos_client().volumes.abort_move(
                vol_id)
            assert abortresp is not None
    except exceptions.NimOSAPIError as ex:
        if ("SM_vol_connection_count_unavailable" in str(ex)
                or "SM_vol_usage_unavailable" in str(ex)):
            log("Failed to abort volume move due to service inavailiability")
        elif "SM_invalid_arg_value" in str(ex):
            log("Failed as expected. Invalid pool id provided")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_bulk_set_dedupe(setup_teardown_for_each_test):
    try:
        # first atleast create volume
        create_volume(vol_name3, size=5)
        dedupresp = nimosclientbase.get_nimos_client().volumes.bulk_set_dedupe(
            dedupe_enabled=True, vol_ids=vol_to_delete)
        assert dedupresp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_pool_dedupe_incapable" in str(ex):
            log("Failed as expected. pool is not"
                "capable of hosting dedup volumes")
        elif "Object is already in requested state" in str(ex):
            log("Failed as expected. Object already in that state.")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_restore_online_volume(setup_teardown_for_each_test):
    try:
        snapshot_resp = None
        # first create volume
        vol_resp = create_volume(vol_name1, size=5)
        snapshot_name1 = nimosclientbase.get_unique_string("volumetc-snapshot1")
        snapshot_resp = nimosclientbase.get_nimos_client().snapshots.create(name=snapshot_name1,
                                                                            vol_id=vol_to_delete[0],
                                                                            description="created by testcase",
                                                                            online=False,
                                                                            writable=False)
        assert snapshot_resp is not None
        # restore the volume
        vol_restore_resp = nimosclientbase.get_nimos_client().volumes.restore(
            base_snap_id=snapshot_resp.attrs.get("id"),
            id=vol_resp.attrs.get("id"))
        assert vol_restore_resp is not None
    except exceptions.NimOSAPIError as ex:
        # fiji and below throws SM_eBusy as exception
        if "SM_vol_not_offline_on_restore" in str(ex) or "SM_ebusy" in str(ex):
            log("Failed as expected. volume to restore is not offline")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_restore_offline_volume(setup_teardown_for_each_test):
    try:
        # first create volume
        vol_resp = create_volume(vol_name1, size=5)
        snapshot_name1 = nimosclientbase.get_unique_string("volumetc-snapshot1")
        snapshot_resp = nimosclientbase.get_nimos_client().snapshots.create(name=snapshot_name1,
                                                                            vol_id=vol_to_delete[0],
                                                                            description="created by testcase",
                                                                            online=False,
                                                                            writable=False)
        assert snapshot_resp is not None
        # offline and restore
        nimosclientbase.get_nimos_client().volumes.offline(id=vol_resp.attrs.get("id"))
        vol_restore_resp = nimosclientbase.get_nimos_client().volumes.restore(
            base_snap_id=snapshot_resp.attrs.get("id"),
            id=vol_resp.attrs.get("id"))
        assert vol_restore_resp is not None
    except exceptions.NimOSAPIError as ex:
        log(f"Failed with exception message : {str(ex)}")
        raise ex



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author mandar shivrekar

import pytest
import os
import nimbleclient
from tests.nimbleclientbase import SKIPTEST, log_to_file as log

'''Version check test for NimOS package'''

@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")

def test_nimos_sdk_package_versioning():
    log("**** Starting Tests for NimOS SDK package versioning TestCase *****\n")

    version = None
    with open(nimbleclient.__file__) as fh:
        info = fh.read()
        version = [line for line in info.split('\n') if line.startswith('__version__')][0].split(' = ')[1][1:-1]
        log("Obtained {!r} version for NimOS SDK package".format(version))
    assert(version is not None)

    log("**** Completed Tests for NimOS SDK package versioning TestCase *****\n")




# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

import pytest

import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions


'''FCConfigTestCases class tests the fibre channel functionality.
    It covers ports,session,initiator_aliases and interface
    object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for FibreChannel Config TestCases *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for FibreChannel Config TestCases *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    nimosclientbase.log_footer(request.function.__name__)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_fc_configs(setup_teardown_for_each_test):
    nimosclientbase.get_nimos_client().fibre_channel_configs.get()


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_fcconfigs_endrow_beyond(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().fibre_channel_configs.get(
            endRow=30)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_end_row_beyond_total_rows" in str(ex):
            log("Failed as expected.no rows present")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_select_fields_for_fcconfigs(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().fibre_channel_configs.get(
            fields="id,group_leader_array")
        assert resp is not None
        assert resp.attrs.get("id") is not None
        assert resp.attrs.get("group_leader_array") is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_end_row_beyond_total_rows" in str(ex):
            log("Failed as expected")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class FibreChannelPort(Resource):
    """Fibre Channel ports provide data access. This API provides the list of all Fibre Channel ports configured on the arrays.

    # Parameters
    id                   : Identifier for the Fibre Channel port.
    array_name_or_serial : Name or serial number of the array.
    controller_name      : Name (A or B) of the controller to which the port belongs.
    fc_port_name         : Name of the Fibre Channel port.
    bus_location         : PCI bus location of the HBA (Host Bus Adapter) for this Fibre Channel port.
    port                 : HBA (Host Bus Adapter) port number for this Fibre Channel port.
    slot                 : HBA (Host Bus Adapter) slot number for this Fibre Channel port.
    orientation          : Orientation of FC ports on a HBA. An orientation of 'right_to_left' indicates that ports are ordered as 3,2,1,0 on the slot. Possible values:
                           'left_to_right', 'right_to_left'.
    link_info            : Information about the Fibre Channel link associated with this port.
    rx_power             : SFP RX power in uW.
    tx_power             : SFP TX power in uW.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class FibreChannelPortList(Collection):
    resource = FibreChannelPort
    resource_type = "fibre_channel_ports"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class FibreChannelConfig(Resource):
    """Manage group wide Fibre Channel configuration.

    # Parameters
    id                 : Identifier for Fibre Channel configuration.
    array_list         : List of array Fibre Channel configs.
    group_leader_array : Name of the group leader array.
    """
    def regenerate(self, precheck, wwnn_base_str, **kwargs):
        """Regenerate Fibre Channel configuration.

        # Parameters
        id            : ID of the Fibre Channel configuration.
        wwnn_base_str : Base World Wide Node Name(WWNN).
        precheck      : Check if the interfaces are offline before regenerating the WWNN (World Wide Node Name).
        """

        return self._collection.regenerate(
            self.id,
            precheck,
            wwnn_base_str,
            **kwargs
        )
    def hw_upgrade(self, **kwargs):
        """Update Fibre Channel configuration after hardware changes.

        # Parameters
        id : ID of the Fibre Channel configuration.
        """

        return self._collection.hw_upgrade(
            self.id,
            **kwargs
        )

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class FibreChannelConfigList(Collection):
    resource = FibreChannelConfig
    resource_type = "fibre_channel_configs"

    def regenerate(self, id, precheck, wwnn_base_str, **kwargs):
        """Regenerate Fibre Channel configuration.

        # Parameters
        id            : ID of the Fibre Channel configuration.
        wwnn_base_str : Base World Wide Node Name(WWNN).
        precheck      : Check if the interfaces are offline before regenerating the WWNN (World Wide Node Name).
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'regenerate',
            id=id,
            precheck=precheck,
            wwnn_base_str=wwnn_base_str,
            **kwargs
        )

    def hw_upgrade(self, id, **kwargs):
        """Update Fibre Channel configuration after hardware changes.

        # Parameters
        id : ID of the Fibre Channel configuration.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'hw_upgrade',
            id=id,
            **kwargs
        )

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan
import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions


'''DisksTestCase tests the disks object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for Disks TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for Disks TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    nimosclientbase.log_footer(request.function.__name__)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_disks(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().disks.get()
    assert resp is not None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_update_disks_mandatory_params(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().disks.get()
    assert resp is not None
    # update
    try:
        nimosclientbase.get_nimos_client().disks.update(
            id=resp.attrs.get("id"),
            force=False)
    except exceptions.NimOSAPIError as ex:
        if "SM_missing_arg" in str(ex):
            log("Failed as expected. mandatory param 'disk_op' not provided")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_disks_query_params(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().disks.list()
    assert resp is not None
    resp = nimosclientbase.get_nimos_client().disks.get(
        array_id=resp[0].attrs.get("array_id"))
    assert resp is not None
    # assert that those fields are present
    assert resp.attrs.get("array_id") is not None
    # try asserting a value which was not querying
    assert resp.attrs.get("startRow") is None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_query_invalid_params(setup_teardown_for_each_test):
    try:
        query_param = "activity"
        resp = nimosclientbase.get_nimos_client().disks.get(
            limit=2,
            fields=query_param)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_unexpected_query_param" in str(ex):
            log("Failed as expected. "
                f"Invalid query params provided to query: '{query_param }'")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class Shelf(Resource):
    """Disk shelf and head unit houses disks and controller.

    # Parameters
    id                   : ID of shelf.
    array_name           : Name of array the shelf belongs to.
    array_id             : ID of array the shelf belongs to.
    partial_response_ok  : Indicate that it is okay to provide partially available response.
    chassis_type         : Chassis type.
    ctrlrs               : List of ctrlr info.
    serial               : The serial number of the chassis.
    model                : Model of the shelf or head unit.
    model_ext            : Extended model of the shelf or head unit.
    chassis_sensors      : List of chassis sensor readings.
    psu_overall_status   : The overall status for the PSUs.
    fan_overall_status   : The overall status for the fans on both controllers.
    temp_overall_status  : The overall status for the temperature on both controllers.
    disk_sets            : Attributes for the disk sets in this shelf.
    activated            : Activated state for shelf or disk set means it is available to store date on. An activated shelf may not be deactivated.
    driveset             : Driveset to activate.
    force                : Forcibly activate shelf.
    accept_foreign       : Accept the removal of data on the shelf disks and activate foreign shelf.
    accept_dedupe_impact : Accept the reduction or elimination of deduplication capability on the system as a result of activating a shelf that does not meet the necessary
                           deduplication requirements.
    last_request         : Indicates this is the last request in a series of shelf add requests.
    """
    def identify(self, cid, status, **kwargs):
        """Turn on chassis identifier for a controller.

        # Parameters
        id     : ID of shelf.
        cid    : Possible values: 'A', 'B'.
        status : Status value of identifier to set.
        """

        return self._collection.identify(
            self.id,
            cid,
            status,
            **kwargs
        )
    def evacuate(self, driveset, **kwargs):
        """Perform shelf evacuation.

        # Parameters
        id       : ID of shelf.
        driveset : Driveset to evacuate.
        dry_run  : Argument to perform a dry run, not the actual shelf evacuation.
        start    : Argument to perform the shelf evacuation.
        cancel   : Argument to cancel the shelf evacuation.
        pause    : Argument to pause the shelf evacuation.
        resume   : Argument to resume the shelf evacuation.
        """

        return self._collection.evacuate(
            self.id,
            driveset,
            **kwargs
        )

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")


class ShelfList(Collection):
    resource = Shelf
    resource_type = "shelves"

    def identify(self, id, cid, status, **kwargs):
        """Turn on chassis identifier for a controller.

        # Parameters
        id     : ID of shelf.
        cid    : Possible values: 'A', 'B'.
        status : Status value of identifier to set.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'identify',
            cid=cid,
            id=id,
            status=status,
            **kwargs
        )

    def evacuate(self, id, driveset, **kwargs):
        """Perform shelf evacuation.

        # Parameters
        id       : ID of shelf.
        driveset : Driveset to evacuate.
        dry_run  : Argument to perform a dry run, not the actual shelf evacuation.
        start    : Argument to perform the shelf evacuation.
        cancel   : Argument to cancel the shelf evacuation.
        pause    : Argument to pause the shelf evacuation.
        resume   : Argument to resume the shelf evacuation.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'evacuate',
            driveset=driveset,
            id=id,
            **kwargs
        )

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP
#
# @author alok ranjan
from tests.nimbleclientbase import SKIPTEST, log_to_file as log


'''AlarmsTestCase is not implemented due to external dependency '''


log("**** AlarmsTestCase is not implemented due to external dependency *****'")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#

from ..query import _Field


class ApplicationServerFields():
    id = _Field('id')
    name = _Field('name')
    hostname = _Field('hostname')
    port = _Field('port')
    username = _Field('username')
    description = _Field('description')
    password = _Field('password')
    server_type = _Field('server_type')

    @classmethod
    def metadata(cls, key):
        return _Field('metadata.' + key)

    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')


class DiskFields():
    id = _Field('id')
    is_dfc = _Field('is_dfc')
    serial = _Field('serial')
    path = _Field('path')
    shelf_serial = _Field('shelf_serial')
    shelf_location = _Field('shelf_location')
    shelf_id = _Field('shelf_id')
    shelf_location_id = _Field('shelf_location_id')
    vshelf_id = _Field('vshelf_id')
    slot = _Field('slot')
    bank = _Field('bank')
    model = _Field('model')
    vendor = _Field('vendor')
    firmware_version = _Field('firmware_version')
    hba = _Field('hba')
    port = _Field('port')
    size = _Field('size')
    state = _Field('state')
    type = _Field('type')
    block_type = _Field('block_type')
    raid_id = _Field('raid_id')
    raid_resync_percent = _Field('raid_resync_percent')
    raid_resync_current_speed = _Field('raid_resync_current_speed')
    raid_resync_average_speed = _Field('raid_resync_average_speed')
    raid_state = _Field('raid_state')
    disk_internal_stat_1 = _Field('disk_internal_stat_1')
    smart_attribute_list = _Field('smart_attribute_list')
    disk_op = _Field('disk_op')
    force = _Field('force')
    array_name = _Field('array_name')
    array_id = _Field('array_id')
    partial_response_ok = _Field('partial_response_ok')


class FolderFields():
    id = _Field('id')
    name = _Field('name')
    fqn = _Field('fqn')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    pool_name = _Field('pool_name')
    pool_id = _Field('pool_id')
    limit_bytes_specified = _Field('limit_bytes_specified')
    limit_bytes = _Field('limit_bytes')
    limit_size_bytes = _Field('limit_size_bytes')
    provisioned_limit_size_bytes = _Field('provisioned_limit_size_bytes')
    overdraft_limit_pct = _Field('overdraft_limit_pct')
    capacity_bytes = _Field('capacity_bytes')
    free_space_bytes = _Field('free_space_bytes')
    provisioned_bytes = _Field('provisioned_bytes')
    usage_bytes = _Field('usage_bytes')
    volume_mapped_bytes = _Field('volume_mapped_bytes')
    usage_valid = _Field('usage_valid')
    agent_type = _Field('agent_type')
    inherited_vol_perfpol_id = _Field('inherited_vol_perfpol_id')
    inherited_vol_perfpol_name = _Field('inherited_vol_perfpol_name')
    unused_reserve_bytes = _Field('unused_reserve_bytes')
    unused_snap_reserve_bytes = _Field('unused_snap_reserve_bytes')
    compressed_vol_usage_bytes = _Field('compressed_vol_usage_bytes')
    compressed_snap_usage_bytes = _Field('compressed_snap_usage_bytes')
    uncompressed_vol_usage_bytes = _Field('uncompressed_vol_usage_bytes')
    uncompressed_snap_usage_bytes = _Field('uncompressed_snap_usage_bytes')
    vol_compression_ratio = _Field('vol_compression_ratio')
    snap_compression_ratio = _Field('snap_compression_ratio')
    compression_ratio = _Field('compression_ratio')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    num_snaps = _Field('num_snaps')
    num_snapcolls = _Field('num_snapcolls')
    app_uuid = _Field('app_uuid')
    volume_list = _Field('volume_list')
    appserver_id = _Field('appserver_id')
    appserver_name = _Field('appserver_name')
    folset_id = _Field('folset_id')
    folset_name = _Field('folset_name')
    limit_iops = _Field('limit_iops')
    limit_mbps = _Field('limit_mbps')
    access_protocol = _Field('access_protocol')
    tenant_id = _Field('tenant_id')


class InitiatorFields():
    id = _Field('id')
    access_protocol = _Field('access_protocol')
    initiator_group_id = _Field('initiator_group_id')
    initiator_group_name = _Field('initiator_group_name')
    label = _Field('label')
    iqn = _Field('iqn')
    ip_address = _Field('ip_address')
    alias = _Field('alias')
    chapuser_id = _Field('chapuser_id')
    wwpn = _Field('wwpn')
    vp_override = _Field('vp_override')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    override_existing_alias = _Field('override_existing_alias')


class ActiveDirectoryMembershipFields():
    id = _Field('id')
    description = _Field('description')
    name = _Field('name')
    netbios = _Field('netbios')
    server_list = _Field('server_list')
    computer_name = _Field('computer_name')
    organizational_unit = _Field('organizational_unit')
    user = _Field('user')
    password = _Field('password')
    enabled = _Field('enabled')


class PerformancePolicyFields():
    id = _Field('id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    block_size = _Field('block_size')
    compress = _Field('compress')
    cache = _Field('cache')
    cache_policy = _Field('cache_policy')
    space_policy = _Field('space_policy')
    app_category = _Field('app_category')
    dedupe_enabled = _Field('dedupe_enabled')
    deprecated = _Field('deprecated')
    predefined = _Field('predefined')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    sample_rate = _Field('sample_rate')
    volume_count = _Field('volume_count')
    dedupe_override_pools = _Field('dedupe_override_pools')


class ChapUserFields():
    id = _Field('id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    password = _Field('password')
    initiator_iqns = _Field('initiator_iqns')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    vol_list = _Field('vol_list')
    vol_count = _Field('vol_count')


class AuditLogFields():
    id = _Field('id')
    type = _Field('type')
    object_id = _Field('object_id')
    object_name = _Field('object_name')
    object_type = _Field('object_type')
    scope = _Field('scope')
    time = _Field('time')
    status = _Field('status')
    error_code = _Field('error_code')
    user_id = _Field('user_id')
    user_name = _Field('user_name')
    user_full_name = _Field('user_full_name')
    source_ip = _Field('source_ip')
    ext_user_id = _Field('ext_user_id')
    ext_user_group_id = _Field('ext_user_group_id')
    ext_user_group_name = _Field('ext_user_group_name')
    app_name = _Field('app_name')
    access_type = _Field('access_type')
    category = _Field('category')
    activity_type = _Field('activity_type')
    activity = _Field('activity')


class FibreChannelPortFields():
    id = _Field('id')
    array_name_or_serial = _Field('array_name_or_serial')
    controller_name = _Field('controller_name')
    fc_port_name = _Field('fc_port_name')
    bus_location = _Field('bus_location')
    port = _Field('port')
    slot = _Field('slot')
    orientation = _Field('orientation')
    link_info = _Field('link_info')
    rx_power = _Field('rx_power')
    tx_power = _Field('tx_power')


class FibreChannelConfigFields():
    id = _Field('id')
    array_list = _Field('array_list')
    group_leader_array = _Field('group_leader_array')


class SnapshotCollectionFields():
    id = _Field('id')
    name = _Field('name')
    description = _Field('description')
    volcoll_name = _Field('volcoll_name')
    volcoll_id = _Field('volcoll_id')
    origin_name = _Field('origin_name')
    is_replica = _Field('is_replica')
    srep_owner_name = _Field('srep_owner_name')
    srep_owner_id = _Field('srep_owner_id')
    peer_snapcoll_id = _Field('peer_snapcoll_id')
    is_complete = _Field('is_complete')
    is_manual = _Field('is_manual')
    is_external_trigger = _Field('is_external_trigger')
    is_unmanaged = _Field('is_unmanaged')
    is_manually_managed = _Field('is_manually_managed')
    repl_status = _Field('repl_status')
    repl_start_time = _Field('repl_start_time')
    repl_complete_time = _Field('repl_complete_time')
    repl_bytes_transferred = _Field('repl_bytes_transferred')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    online_status = _Field('online_status')
    vol_snap_attr_list = _Field('vol_snap_attr_list')
    snapshots_list = _Field('snapshots_list')
    replicate = _Field('replicate')
    replicate_to = _Field('replicate_to')
    start_online = _Field('start_online')
    allow_writes = _Field('allow_writes')
    disable_appsync = _Field('disable_appsync')
    snap_verify = _Field('snap_verify')
    skip_db_consistency_check = _Field('skip_db_consistency_check')
    sched_id = _Field('sched_id')
    sched_name = _Field('sched_name')
    invoke_on_upstream_partner = _Field('invoke_on_upstream_partner')
    agent_type = _Field('agent_type')
    expiry_after = _Field('expiry_after')

    @classmethod
    def metadata(cls, key):
        return _Field('metadata.' + key)

    force = _Field('force')


class TokenFields():
    id = _Field('id')
    session_token = _Field('session_token')
    username = _Field('username')
    password = _Field('password')
    app_name = _Field('app_name')
    sdk_name = _Field('sdk_name')
    source_ip = _Field('source_ip')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    expiry_time = _Field('expiry_time')
    server_uuid = _Field('server_uuid')


class ApplicationCategoryFields():
    id = _Field('id')
    name = _Field('name')
    dedupe_enabled = _Field('dedupe_enabled')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')


class ArrayFields():
    id = _Field('id')
    name = _Field('name')
    force = _Field('force')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    status = _Field('status')
    role = _Field('role')
    group_state = _Field('group_state')
    pool_name = _Field('pool_name')
    pool_id = _Field('pool_id')
    model = _Field('model')
    serial = _Field('serial')
    version = _Field('version')
    is_sfa = _Field('is_sfa')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    usage_valid = _Field('usage_valid')
    usable_capacity_bytes = _Field('usable_capacity_bytes')
    usable_cache_capacity_bytes = _Field('usable_cache_capacity_bytes')
    raw_capacity_bytes = _Field('raw_capacity_bytes')
    vol_usage_bytes = _Field('vol_usage_bytes')
    vol_usage_uncompressed_bytes = _Field('vol_usage_uncompressed_bytes')
    vol_compression = _Field('vol_compression')
    vol_saved_bytes = _Field('vol_saved_bytes')
    snap_usage_bytes = _Field('snap_usage_bytes')
    snap_usage_uncompressed_bytes = _Field('snap_usage_uncompressed_bytes')
    snap_compression = _Field('snap_compression')
    snap_space_reduction = _Field('snap_space_reduction')
    snap_saved_bytes = _Field('snap_saved_bytes')
    pending_delete_bytes = _Field('pending_delete_bytes')
    available_bytes = _Field('available_bytes')
    usage = _Field('usage')
    all_flash = _Field('all_flash')
    dedupe_capacity_bytes = _Field('dedupe_capacity_bytes')
    dedupe_usage_bytes = _Field('dedupe_usage_bytes')
    is_fully_dedupe_capable = _Field('is_fully_dedupe_capable')
    extended_model = _Field('extended_model')
    is_supported_hw_config = _Field('is_supported_hw_config')
    gig_nic_port_count = _Field('gig_nic_port_count')
    ten_gig_sfp_nic_port_count = _Field('ten_gig_sfp_nic_port_count')
    ten_gig_t_nic_port_count = _Field('ten_gig_t_nic_port_count')
    fc_port_count = _Field('fc_port_count')
    public_key = _Field('public_key')
    upgrade = _Field('upgrade')
    create_pool = _Field('create_pool')
    pool_description = _Field('pool_description')
    allow_lower_limits = _Field('allow_lower_limits')
    ctrlr_a_support_ip = _Field('ctrlr_a_support_ip')
    ctrlr_b_support_ip = _Field('ctrlr_b_support_ip')
    nic_list = _Field('nic_list')
    model_sub_type = _Field('model_sub_type')
    zconf_ipaddrs = _Field('zconf_ipaddrs')
    secondary_mgmt_ip = _Field('secondary_mgmt_ip')


class EventFields():
    id = _Field('id')
    type = _Field('type')
    name = _Field('name')
    scope = _Field('scope')
    target = _Field('target')
    target_type = _Field('target_type')
    timestamp = _Field('timestamp')
    category = _Field('category')
    severity = _Field('severity')
    summary = _Field('summary')
    activity = _Field('activity')
    alarm_id = _Field('alarm_id')
    params = _Field('params')
    tenant_id = _Field('tenant_id')


class SoftwareVersionFields():
    version = _Field('version')
    signature = _Field('signature')
    name = _Field('name')
    status = _Field('status')
    total_bytes = _Field('total_bytes')
    downloaded_bytes = _Field('downloaded_bytes')
    blacklist_reason = _Field('blacklist_reason')
    release_date = _Field('release_date')
    is_manually_downloaded = _Field('is_manually_downloaded')
    release_status = _Field('release_status')
    no_partial_response = _Field('no_partial_response')


class UserGroupFields():
    id = _Field('id')
    name = _Field('name')
    description = _Field('description')
    role_id = _Field('role_id')
    role = _Field('role')
    inactivity_timeout = _Field('inactivity_timeout')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    disabled = _Field('disabled')
    external_id = _Field('external_id')
    domain_id = _Field('domain_id')
    domain_name = _Field('domain_name')


class JobFields():
    completion_time = _Field('completion_time')
    creation_time = _Field('creation_time')
    current_phase = _Field('current_phase')
    current_phase_description = _Field('current_phase_description')
    description = _Field('description')
    id = _Field('id')
    name = _Field('name')
    last_modified = _Field('last_modified')
    object_id = _Field('object_id')
    op_type = _Field('op_type')
    type = _Field('type')
    parent_job_id = _Field('parent_job_id')
    percent_complete = _Field('percent_complete')
    request = _Field('request')
    response = _Field('response')
    state = _Field('state')
    result = _Field('result')
    total_phases = _Field('total_phases')


class ReplicationPartnerFields():
    id = _Field('id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    partner_type = _Field('partner_type')
    alias = _Field('alias')
    secret = _Field('secret')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    control_port = _Field('control_port')
    hostname = _Field('hostname')
    port_range_start = _Field('port_range_start')
    proxy_hostname = _Field('proxy_hostname')
    proxy_user = _Field('proxy_user')
    repl_hostname = _Field('repl_hostname')
    data_port = _Field('data_port')
    is_alive = _Field('is_alive')
    partner_group_uid = _Field('partner_group_uid')
    last_keepalive_error = _Field('last_keepalive_error')
    cfg_sync_status = _Field('cfg_sync_status')
    last_sync_error = _Field('last_sync_error')
    array_serial = _Field('array_serial')
    version = _Field('version')
    pool_id = _Field('pool_id')
    pool_name = _Field('pool_name')
    folder_id = _Field('folder_id')
    folder_name = _Field('folder_name')
    match_folder = _Field('match_folder')
    paused = _Field('paused')
    unique_name = _Field('unique_name')
    subnet_label = _Field('subnet_label')
    subnet_type = _Field('subnet_type')
    throttles = _Field('throttles')
    throttled_bandwidth = _Field('throttled_bandwidth')
    throttled_bandwidth_current = _Field('throttled_bandwidth_current')
    throttled_bandwidth_kbps = _Field('throttled_bandwidth_kbps')
    throttled_bandwidth_current_kbps = _Field('throttled_bandwidth_current_kbps')
    subnet_network = _Field('subnet_network')
    subnet_netmask = _Field('subnet_netmask')
    volume_collection_list = _Field('volume_collection_list')
    volume_collection_list_count = _Field('volume_collection_list_count')
    volume_list = _Field('volume_list')
    volume_list_count = _Field('volume_list_count')
    replication_direction = _Field('replication_direction')


class InitiatorGroupFields():
    id = _Field('id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    access_protocol = _Field('access_protocol')
    host_type = _Field('host_type')
    fc_tdz_ports = _Field('fc_tdz_ports')
    target_subnets = _Field('target_subnets')
    iscsi_initiators = _Field('iscsi_initiators')
    fc_initiators = _Field('fc_initiators')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    vp_override = _Field('vp_override')
    app_uuid = _Field('app_uuid')
    volume_count = _Field('volume_count')
    volume_list = _Field('volume_list')
    num_connections = _Field('num_connections')

    @classmethod
    def metadata(cls, key):
        return _Field('metadata.' + key)


class WitnessFields():
    id = _Field('id')
    username = _Field('username')
    password = _Field('password')
    host = _Field('host')
    port = _Field('port')
    secure_mode = _Field('secure_mode')
    auto_switchover_messages = _Field('auto_switchover_messages')


class SubnetFields():
    id = _Field('id')
    name = _Field('name')
    network = _Field('network')
    netmask = _Field('netmask')
    type = _Field('type')
    allow_iscsi = _Field('allow_iscsi')
    allow_group = _Field('allow_group')
    discovery_ip = _Field('discovery_ip')
    mtu = _Field('mtu')
    netzone_type = _Field('netzone_type')
    vlan_id = _Field('vlan_id')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    failover = _Field('failover')
    failover_enable_time = _Field('failover_enable_time')


class GroupFields():
    id = _Field('id')
    name = _Field('name')
    smtp_server = _Field('smtp_server')
    smtp_port = _Field('smtp_port')
    smtp_auth_enabled = _Field('smtp_auth_enabled')
    smtp_auth_username = _Field('smtp_auth_username')
    smtp_auth_password = _Field('smtp_auth_password')
    smtp_encrypt_type = _Field('smtp_encrypt_type')
    autosupport_enabled = _Field('autosupport_enabled')
    allow_analytics_gui = _Field('allow_analytics_gui')
    allow_support_tunnel = _Field('allow_support_tunnel')
    proxy_server = _Field('proxy_server')
    proxy_port = _Field('proxy_port')
    proxy_username = _Field('proxy_username')
    proxy_password = _Field('proxy_password')
    alert_to_email_addrs = _Field('alert_to_email_addrs')
    send_alert_to_support = _Field('send_alert_to_support')
    alert_from_email_addr = _Field('alert_from_email_addr')
    alert_min_level = _Field('alert_min_level')
    isns_enabled = _Field('isns_enabled')
    isns_server = _Field('isns_server')
    isns_port = _Field('isns_port')
    snmp_trap_enabled = _Field('snmp_trap_enabled')
    snmp_trap_host = _Field('snmp_trap_host')
    snmp_trap_port = _Field('snmp_trap_port')
    snmp_get_enabled = _Field('snmp_get_enabled')
    snmp_community = _Field('snmp_community')
    snmp_get_port = _Field('snmp_get_port')
    snmp_sys_contact = _Field('snmp_sys_contact')
    snmp_sys_location = _Field('snmp_sys_location')
    domain_name = _Field('domain_name')
    dns_servers = _Field('dns_servers')
    ntp_server = _Field('ntp_server')
    timezone = _Field('timezone')
    user_inactivity_timeout = _Field('user_inactivity_timeout')
    syslogd_enabled = _Field('syslogd_enabled')
    syslogd_server = _Field('syslogd_server')
    syslogd_port = _Field('syslogd_port')
    vvol_enabled = _Field('vvol_enabled')
    iscsi_enabled = _Field('iscsi_enabled')
    fc_enabled = _Field('fc_enabled')
    unique_name_enabled = _Field('unique_name_enabled')
    access_protocol_list = _Field('access_protocol_list')
    group_target_enabled = _Field('group_target_enabled')
    default_iscsi_target_scope = _Field('default_iscsi_target_scope')
    tdz_enabled = _Field('tdz_enabled')
    tdz_prefix = _Field('tdz_prefix')
    group_target_name = _Field('group_target_name')
    default_volume_reserve = _Field('default_volume_reserve')
    default_volume_warn_level = _Field('default_volume_warn_level')
    default_volume_limit = _Field('default_volume_limit')
    default_snap_reserve = _Field('default_snap_reserve')
    default_snap_warn_level = _Field('default_snap_warn_level')
    default_snap_limit = _Field('default_snap_limit')
    default_snap_limit_percent = _Field('default_snap_limit_percent')
    alarms_enabled = _Field('alarms_enabled')
    vss_validation_timeout = _Field('vss_validation_timeout')
    auto_switchover_enabled = _Field('auto_switchover_enabled')
    auto_switchover_messages = _Field('auto_switchover_messages')
    merge_state = _Field('merge_state')
    merge_group_name = _Field('merge_group_name')
    tlsv1_enabled = _Field('tlsv1_enabled')
    cc_mode_enabled = _Field('cc_mode_enabled')
    group_snapshot_ttl = _Field('group_snapshot_ttl')
    autoclean_unmanaged_snapshots_ttl_unit = _Field('autoclean_unmanaged_snapshots_ttl_unit')
    autoclean_unmanaged_snapshots_enabled = _Field('autoclean_unmanaged_snapshots_enabled')
    leader_array_name = _Field('leader_array_name')
    leader_array_serial = _Field('leader_array_serial')
    management_service_backup_array_name = _Field('management_service_backup_array_name')
    management_service_backup_status = _Field('management_service_backup_status')
    failover_mode = _Field('failover_mode')
    witness_status = _Field('witness_status')
    member_list = _Field('member_list')
    compressed_vol_usage_bytes = _Field('compressed_vol_usage_bytes')
    compressed_snap_usage_bytes = _Field('compressed_snap_usage_bytes')
    uncompressed_vol_usage_bytes = _Field('uncompressed_vol_usage_bytes')
    uncompressed_snap_usage_bytes = _Field('uncompressed_snap_usage_bytes')
    usable_capacity_bytes = _Field('usable_capacity_bytes')
    usage = _Field('usage')
    raw_capacity = _Field('raw_capacity')
    usable_cache_capacity = _Field('usable_cache_capacity')
    raw_cache_capacity = _Field('raw_cache_capacity')
    snap_usage_populated = _Field('snap_usage_populated')
    pending_deletes = _Field('pending_deletes')
    num_connections = _Field('num_connections')
    vol_compression_ratio = _Field('vol_compression_ratio')
    snap_compression_ratio = _Field('snap_compression_ratio')
    compression_ratio = _Field('compression_ratio')
    dedupe_ratio = _Field('dedupe_ratio')
    clone_ratio = _Field('clone_ratio')
    vol_thin_provisioning_ratio = _Field('vol_thin_provisioning_ratio')
    savings_ratio = _Field('savings_ratio')
    data_reduction_ratio = _Field('data_reduction_ratio')
    savings_dedupe = _Field('savings_dedupe')
    savings_compression = _Field('savings_compression')
    savings_clone = _Field('savings_clone')
    savings_vol_thin_provisioning = _Field('savings_vol_thin_provisioning')
    savings_data_reduction = _Field('savings_data_reduction')
    savings = _Field('savings')
    free_space = _Field('free_space')
    unused_reserve_bytes = _Field('unused_reserve_bytes')
    usage_valid = _Field('usage_valid')
    space_info_valid = _Field('space_info_valid')
    version_current = _Field('version_current')
    version_target = _Field('version_target')
    version_rollback = _Field('version_rollback')
    update_state = _Field('update_state')
    update_start_time = _Field('update_start_time')
    update_end_time = _Field('update_end_time')
    update_array_names = _Field('update_array_names')
    update_progress_msg = _Field('update_progress_msg')
    update_error_code = _Field('update_error_code')
    update_downloading = _Field('update_downloading')
    update_download_error_code = _Field('update_download_error_code')
    update_download_start_time = _Field('update_download_start_time')
    update_download_end_time = _Field('update_download_end_time')
    iscsi_automatic_connection_method = _Field('iscsi_automatic_connection_method')
    iscsi_connection_rebalancing = _Field('iscsi_connection_rebalancing')
    repl_throttled_bandwidth = _Field('repl_throttled_bandwidth')
    repl_throttled_bandwidth_kbps = _Field('repl_throttled_bandwidth_kbps')
    repl_throttle_list = _Field('repl_throttle_list')
    volume_migration_status = _Field('volume_migration_status')
    array_unassign_migration_status = _Field('array_unassign_migration_status')
    data_rebalance_status = _Field('data_rebalance_status')
    scsi_vendor_id = _Field('scsi_vendor_id')
    encryption_config = _Field('encryption_config')
    last_login = _Field('last_login')
    num_snaps = _Field('num_snaps')
    num_snapcolls = _Field('num_snapcolls')
    date = _Field('date')
    login_banner_message = _Field('login_banner_message')
    login_banner_after_auth = _Field('login_banner_after_auth')
    login_banner_reset = _Field('login_banner_reset')
    snap_retn_meter_high = _Field('snap_retn_meter_high')
    snap_retn_meter_very_high = _Field('snap_retn_meter_very_high')


class AccessControlRecordFields():
    id = _Field('id')
    apply_to = _Field('apply_to')
    chap_user_id = _Field('chap_user_id')
    chap_user_name = _Field('chap_user_name')
    initiator_group_id = _Field('initiator_group_id')
    initiator_group_name = _Field('initiator_group_name')
    lun = _Field('lun')
    vol_id = _Field('vol_id')
    vol_name = _Field('vol_name')
    vol_agent_type = _Field('vol_agent_type')
    pe_id = _Field('pe_id')
    pe_name = _Field('pe_name')
    pe_lun = _Field('pe_lun')
    snap_id = _Field('snap_id')
    snap_name = _Field('snap_name')
    pe_ids = _Field('pe_ids')
    snapluns = _Field('snapluns')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    access_protocol = _Field('access_protocol')


class ShelfFields():
    id = _Field('id')
    array_name = _Field('array_name')
    array_id = _Field('array_id')
    partial_response_ok = _Field('partial_response_ok')
    chassis_type = _Field('chassis_type')
    ctrlrs = _Field('ctrlrs')
    serial = _Field('serial')
    model = _Field('model')
    model_ext = _Field('model_ext')
    chassis_sensors = _Field('chassis_sensors')
    psu_overall_status = _Field('psu_overall_status')
    fan_overall_status = _Field('fan_overall_status')
    temp_overall_status = _Field('temp_overall_status')
    disk_sets = _Field('disk_sets')
    activated = _Field('activated')
    driveset = _Field('driveset')
    force = _Field('force')
    accept_foreign = _Field('accept_foreign')
    accept_dedupe_impact = _Field('accept_dedupe_impact')
    last_request = _Field('last_request')


class FibreChannelInitiatorAliasFields():
    id = _Field('id')
    alias = _Field('alias')
    wwpn = _Field('wwpn')
    source = _Field('source')


class NetworkConfigFields():
    id = _Field('id')
    name = _Field('name')
    mgmt_ip = _Field('mgmt_ip')
    secondary_mgmt_ip = _Field('secondary_mgmt_ip')
    role = _Field('role')
    iscsi_automatic_connection_method = _Field('iscsi_automatic_connection_method')
    iscsi_connection_rebalancing = _Field('iscsi_connection_rebalancing')
    route_list = _Field('route_list')
    subnet_list = _Field('subnet_list')
    array_list = _Field('array_list')
    group_leader_array = _Field('group_leader_array')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    active_since = _Field('active_since')
    last_active = _Field('last_active')
    ignore_validation_mask = _Field('ignore_validation_mask')


class MasterKeyFields():
    id = _Field('id')
    name = _Field('name')
    passphrase = _Field('passphrase')
    new_passphrase = _Field('new_passphrase')
    active = _Field('active')
    purge_age = _Field('purge_age')


class ProtocolEndpointFields():
    id = _Field('id')
    name = _Field('name')
    description = _Field('description')
    pool_name = _Field('pool_name')
    pool_id = _Field('pool_id')
    state = _Field('state')
    serial_number = _Field('serial_number')
    target_name = _Field('target_name')
    group_specific_ids = _Field('group_specific_ids')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    num_connections = _Field('num_connections')
    num_iscsi_connections = _Field('num_iscsi_connections')
    num_fc_connections = _Field('num_fc_connections')
    access_control_records = _Field('access_control_records')
    iscsi_sessions = _Field('iscsi_sessions')
    fc_sessions = _Field('fc_sessions')
    access_protocol = _Field('access_protocol')


class PoolFields():
    id = _Field('id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    capacity = _Field('capacity')
    usage = _Field('usage')
    savings = _Field('savings')
    savings_data_reduction = _Field('savings_data_reduction')
    savings_compression = _Field('savings_compression')
    savings_dedupe = _Field('savings_dedupe')
    savings_clone = _Field('savings_clone')
    savings_vol_thin_provisioning = _Field('savings_vol_thin_provisioning')
    reserve = _Field('reserve')
    unused_reserve = _Field('unused_reserve')
    free_space = _Field('free_space')
    cache_capacity = _Field('cache_capacity')
    pinnable_cache_capacity = _Field('pinnable_cache_capacity')
    pinned_cache_capacity = _Field('pinned_cache_capacity')
    dedupe_capacity_bytes = _Field('dedupe_capacity_bytes')
    dedupe_usage_bytes = _Field('dedupe_usage_bytes')
    savings_ratio = _Field('savings_ratio')
    data_reduction_ratio = _Field('data_reduction_ratio')
    compression_ratio = _Field('compression_ratio')
    dedupe_ratio = _Field('dedupe_ratio')
    clone_ratio = _Field('clone_ratio')
    vol_thin_provisioning_ratio = _Field('vol_thin_provisioning_ratio')
    snapcoll_count = _Field('snapcoll_count')
    snap_count = _Field('snap_count')
    array_count = _Field('array_count')
    vol_count = _Field('vol_count')
    array_list = _Field('array_list')
    unassigned_array_list = _Field('unassigned_array_list')
    vol_list = _Field('vol_list')
    pinned_vol_list = _Field('pinned_vol_list')
    folder_list = _Field('folder_list')
    force = _Field('force')
    usage_valid = _Field('usage_valid')
    uncompressed_vol_usage_bytes = _Field('uncompressed_vol_usage_bytes')
    uncompressed_snap_usage_bytes = _Field('uncompressed_snap_usage_bytes')
    all_flash = _Field('all_flash')
    dedupe_capable = _Field('dedupe_capable')
    dedupe_all_volumes_capable = _Field('dedupe_all_volumes_capable')
    dedupe_all_volumes = _Field('dedupe_all_volumes')
    is_default = _Field('is_default')


class UserFields():
    id = _Field('id')
    name = _Field('name')
    search_name = _Field('search_name')
    description = _Field('description')
    role_id = _Field('role_id')
    role = _Field('role')
    password = _Field('password')
    auth_password = _Field('auth_password')
    inactivity_timeout = _Field('inactivity_timeout')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    full_name = _Field('full_name')
    email_addr = _Field('email_addr')
    disabled = _Field('disabled')
    auth_lock = _Field('auth_lock')
    last_login = _Field('last_login')
    last_logout = _Field('last_logout')
    logged_in = _Field('logged_in')


class SpaceDomainFields():
    id = _Field('id')
    pool_id = _Field('pool_id')
    pool_name = _Field('pool_name')
    app_category_id = _Field('app_category_id')
    app_category_name = _Field('app_category_name')
    perf_policy_names = _Field('perf_policy_names')
    sample_rate = _Field('sample_rate')
    volume_count = _Field('volume_count')
    deduped_volume_count = _Field('deduped_volume_count')
    volumes = _Field('volumes')
    block_size = _Field('block_size')
    deduped = _Field('deduped')
    encrypted = _Field('encrypted')
    usage = _Field('usage')
    vol_logical_usage = _Field('vol_logical_usage')
    snap_logical_usage = _Field('snap_logical_usage')
    vol_mapped_usage = _Field('vol_mapped_usage')
    logical_dedupe_usage = _Field('logical_dedupe_usage')
    physical_dedupe_usage = _Field('physical_dedupe_usage')
    savings_compression = _Field('savings_compression')
    savings_dedupe = _Field('savings_dedupe')
    savings_clone = _Field('savings_clone')
    compressed_usage_bytes = _Field('compressed_usage_bytes')
    uncompressed_usage_bytes = _Field('uncompressed_usage_bytes')
    compression_ratio = _Field('compression_ratio')
    dedupe_ratio = _Field('dedupe_ratio')
    clone_ratio = _Field('clone_ratio')


class UserPolicyFields():
    id = _Field('id')
    allowed_attempts = _Field('allowed_attempts')
    min_length = _Field('min_length')
    upper = _Field('upper')
    lower = _Field('lower')
    digit = _Field('digit')
    special = _Field('special')
    previous_diff = _Field('previous_diff')
    no_reuse = _Field('no_reuse')
    max_sessions = _Field('max_sessions')


class AlarmFields():
    id = _Field('id')
    type = _Field('type')
    array = _Field('array')
    curr_onset_event_id = _Field('curr_onset_event_id')
    object_id = _Field('object_id')
    object_name = _Field('object_name')
    object_type = _Field('object_type')
    onset_time = _Field('onset_time')
    ack_time = _Field('ack_time')
    status = _Field('status')
    user_id = _Field('user_id')
    user_name = _Field('user_name')
    user_full_name = _Field('user_full_name')
    category = _Field('category')
    severity = _Field('severity')
    remind_every = _Field('remind_every')
    remind_every_unit = _Field('remind_every_unit')
    activity = _Field('activity')
    next_notification_time = _Field('next_notification_time')


class ControllerFields():
    id = _Field('id')
    name = _Field('name')
    array_name = _Field('array_name')
    array_id = _Field('array_id')
    partial_response_ok = _Field('partial_response_ok')
    serial = _Field('serial')
    hostname = _Field('hostname')
    support_address = _Field('support_address')
    support_netmask = _Field('support_netmask')
    support_nic = _Field('support_nic')
    power_status = _Field('power_status')
    fan_status = _Field('fan_status')
    temperature_status = _Field('temperature_status')
    power_supplies = _Field('power_supplies')
    fans = _Field('fans')
    temperature_sensors = _Field('temperature_sensors')
    partition_status = _Field('partition_status')
    ctrlr_side = _Field('ctrlr_side')
    state = _Field('state')
    nvme_cards_enabled = _Field('nvme_cards_enabled')
    nvme_cards = _Field('nvme_cards')
    asup_time = _Field('asup_time')


class FibreChannelSessionFields():
    id = _Field('id')
    initiator_info = _Field('initiator_info')
    target_info = _Field('target_info')


class VersionFields():
    name = _Field('name')
    software_version = _Field('software_version')


class VolumeFields():
    id = _Field('id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    size = _Field('size')
    description = _Field('description')
    perfpolicy_name = _Field('perfpolicy_name')
    perfpolicy_id = _Field('perfpolicy_id')
    reserve = _Field('reserve')
    warn_level = _Field('warn_level')
    limit = _Field('limit')
    snap_reserve = _Field('snap_reserve')
    snap_warn_level = _Field('snap_warn_level')
    snap_limit = _Field('snap_limit')
    snap_limit_percent = _Field('snap_limit_percent')
    num_snaps = _Field('num_snaps')
    projected_num_snaps = _Field('projected_num_snaps')
    online = _Field('online')
    owned_by_group = _Field('owned_by_group')
    owned_by_group_id = _Field('owned_by_group_id')
    multi_initiator = _Field('multi_initiator')
    iscsi_target_scope = _Field('iscsi_target_scope')
    pool_name = _Field('pool_name')
    pool_id = _Field('pool_id')
    read_only = _Field('read_only')
    serial_number = _Field('serial_number')
    secondary_serial_number = _Field('secondary_serial_number')
    target_name = _Field('target_name')
    block_size = _Field('block_size')
    offline_reason = _Field('offline_reason')
    clone = _Field('clone')
    parent_vol_name = _Field('parent_vol_name')
    parent_vol_id = _Field('parent_vol_id')
    base_snap_name = _Field('base_snap_name')
    base_snap_id = _Field('base_snap_id')
    replication_role = _Field('replication_role')
    volcoll_name = _Field('volcoll_name')
    volcoll_id = _Field('volcoll_id')
    agent_type = _Field('agent_type')
    force = _Field('force')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    protection_type = _Field('protection_type')
    last_snap = _Field('last_snap')
    last_replicated_snap = _Field('last_replicated_snap')
    dest_pool_name = _Field('dest_pool_name')
    dest_pool_id = _Field('dest_pool_id')
    move_start_time = _Field('move_start_time')
    move_aborting = _Field('move_aborting')
    move_bytes_migrated = _Field('move_bytes_migrated')
    move_bytes_remaining = _Field('move_bytes_remaining')
    move_est_compl_time = _Field('move_est_compl_time')
    usage_valid = _Field('usage_valid')
    space_usage_level = _Field('space_usage_level')
    total_usage_bytes = _Field('total_usage_bytes')
    vol_usage_compressed_bytes = _Field('vol_usage_compressed_bytes')
    vol_usage_uncompressed_bytes = _Field('vol_usage_uncompressed_bytes')
    vol_usage_mapped_bytes = _Field('vol_usage_mapped_bytes')
    snap_usage_compressed_bytes = _Field('snap_usage_compressed_bytes')
    snap_usage_uncompressed_bytes = _Field('snap_usage_uncompressed_bytes')
    snap_usage_populated_bytes = _Field('snap_usage_populated_bytes')
    cache_pinned = _Field('cache_pinned')
    pinned_cache_size = _Field('pinned_cache_size')
    cache_needed_for_pin = _Field('cache_needed_for_pin')
    upstream_cache_pinned = _Field('upstream_cache_pinned')
    cache_policy = _Field('cache_policy')
    thinly_provisioned = _Field('thinly_provisioned')
    vol_state = _Field('vol_state')
    online_snaps = _Field('online_snaps')
    num_connections = _Field('num_connections')
    num_iscsi_connections = _Field('num_iscsi_connections')
    num_fc_connections = _Field('num_fc_connections')
    access_control_records = _Field('access_control_records')
    inherit_acl = _Field('inherit_acl')
    encryption_cipher = _Field('encryption_cipher')
    app_uuid = _Field('app_uuid')
    folder_id = _Field('folder_id')
    folder_name = _Field('folder_name')

    @classmethod
    def metadata(cls, key):
        return _Field('metadata.' + key)

    iscsi_sessions = _Field('iscsi_sessions')
    fc_sessions = _Field('fc_sessions')
    caching_enabled = _Field('caching_enabled')
    previously_deduped = _Field('previously_deduped')
    dedupe_enabled = _Field('dedupe_enabled')
    vpd_t10 = _Field('vpd_t10')
    vpd_ieee0 = _Field('vpd_ieee0')
    vpd_ieee1 = _Field('vpd_ieee1')
    app_category = _Field('app_category')
    limit_iops = _Field('limit_iops')
    limit_mbps = _Field('limit_mbps')
    needs_content_repl = _Field('needs_content_repl')
    content_repl_errors_found = _Field('content_repl_errors_found')
    last_content_snap_br_cg_uid = _Field('last_content_snap_br_cg_uid')
    last_content_snap_br_gid = _Field('last_content_snap_br_gid')
    last_content_snap_id = _Field('last_content_snap_id')
    cksum_last_verified = _Field('cksum_last_verified')
    pre_filter = _Field('pre_filter')
    avg_stats_last_5mins = _Field('avg_stats_last_5mins')
    srep_last_sync = _Field('srep_last_sync')
    srep_resync_percent = _Field('srep_resync_percent')


class ProtectionScheduleFields():
    id = _Field('id')
    name = _Field('name')
    description = _Field('description')
    volcoll_or_prottmpl_type = _Field('volcoll_or_prottmpl_type')
    volcoll_or_prottmpl_id = _Field('volcoll_or_prottmpl_id')
    period = _Field('period')
    period_unit = _Field('period_unit')
    at_time = _Field('at_time')
    until_time = _Field('until_time')
    days = _Field('days')
    num_retain = _Field('num_retain')
    downstream_partner = _Field('downstream_partner')
    downstream_partner_name = _Field('downstream_partner_name')
    downstream_partner_id = _Field('downstream_partner_id')
    upstream_partner_name = _Field('upstream_partner_name')
    upstream_partner_id = _Field('upstream_partner_id')
    replicate_every = _Field('replicate_every')
    num_retain_replica = _Field('num_retain_replica')
    repl_alert_thres = _Field('repl_alert_thres')
    snap_verify = _Field('snap_verify')
    skip_db_consistency_check = _Field('skip_db_consistency_check')
    disable_appsync = _Field('disable_appsync')
    schedule_type = _Field('schedule_type')
    active = _Field('active')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    last_mod_sched_time = _Field('last_mod_sched_time')
    last_replicated_snapcoll_name = _Field('last_replicated_snapcoll_name')
    last_replicated_snapcoll_id = _Field('last_replicated_snapcoll_id')
    last_replicated_at_time = _Field('last_replicated_at_time')
    last_snap_time = _Field('last_snap_time')
    next_snap_time = _Field('next_snap_time')
    next_repl_snap_time = _Field('next_repl_snap_time')
    snap_counter = _Field('snap_counter')
    sched_owner_id = _Field('sched_owner_id')
    sched_owner_name = _Field('sched_owner_name')
    last_config_change_time = _Field('last_config_change_time')
    vol_status_list = _Field('vol_status_list')
    sync_repl_vol_status_list = _Field('sync_repl_vol_status_list')
    use_downstream_for_DR = _Field('use_downstream_for_DR')


class ProtectionTemplateFields():
    id = _Field('id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    repl_priority = _Field('repl_priority')
    app_sync = _Field('app_sync')
    app_server = _Field('app_server')
    app_id = _Field('app_id')
    app_cluster_name = _Field('app_cluster_name')
    app_service_name = _Field('app_service_name')
    vcenter_hostname = _Field('vcenter_hostname')
    vcenter_username = _Field('vcenter_username')
    vcenter_password = _Field('vcenter_password')
    agent_hostname = _Field('agent_hostname')
    agent_username = _Field('agent_username')
    agent_password = _Field('agent_password')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    schedule_list = _Field('schedule_list')


class VolumeCollectionFields():
    id = _Field('id')
    prottmpl_id = _Field('prottmpl_id')
    name = _Field('name')
    full_name = _Field('full_name')
    search_name = _Field('search_name')
    description = _Field('description')
    repl_priority = _Field('repl_priority')
    pol_owner_name = _Field('pol_owner_name')
    replication_type = _Field('replication_type')
    synchronous_replication_type = _Field('synchronous_replication_type')
    synchronous_replication_state = _Field('synchronous_replication_state')
    app_sync = _Field('app_sync')
    app_server = _Field('app_server')
    app_id = _Field('app_id')
    app_cluster_name = _Field('app_cluster_name')
    app_service_name = _Field('app_service_name')
    vcenter_hostname = _Field('vcenter_hostname')
    vcenter_username = _Field('vcenter_username')
    vcenter_password = _Field('vcenter_password')
    agent_hostname = _Field('agent_hostname')
    agent_username = _Field('agent_username')
    agent_password = _Field('agent_password')
    creation_time = _Field('creation_time')
    last_modified_time = _Field('last_modified_time')
    volume_list = _Field('volume_list')
    downstream_volume_list = _Field('downstream_volume_list')
    upstream_volume_list = _Field('upstream_volume_list')
    volume_count = _Field('volume_count')
    cache_pinned_volume_list = _Field('cache_pinned_volume_list')
    last_snapcoll = _Field('last_snapcoll')
    snapcoll_count = _Field('snapcoll_count')
    schedule_list = _Field('schedule_list')
    replication_partner = _Field('replication_partner')
    last_replicated_snapcoll = _Field('last_replicated_snapcoll')
    last_replicated_snapcoll_list = _Field('last_replicated_snapcoll_list')
    protection_type = _Field('protection_type')
    lag_time = _Field('lag_time')
    is_standalone_volcoll = _Field('is_standalone_volcoll')
    total_repl_bytes = _Field('total_repl_bytes')
    repl_bytes_transferred = _Field('repl_bytes_transferred')
    is_handing_over = _Field('is_handing_over')
    handover_replication_partner = _Field('handover_replication_partner')

    @classmethod
    def metadata(cls, key):
        return _Field('metadata.' + key)

    srep_last_sync = _Field('srep_last_sync')
    srep_resync_percent = _Field('srep_resync_percent')


class FibreChannelInterfaceFields():
    id = _Field('id')
    array_name_or_serial = _Field('array_name_or_serial')
    partial_response_ok = _Field('partial_response_ok')
    controller_name = _Field('controller_name')
    fc_port_id = _Field('fc_port_id')
    name = _Field('name')
    wwnn = _Field('wwnn')
    wwpn = _Field('wwpn')
    peerzone = _Field('peerzone')
    online = _Field('online')
    firmware_version = _Field('firmware_version')
    logical_port_number = _Field('logical_port_number')
    fc_port_name = _Field('fc_port_name')
    bus_location = _Field('bus_location')
    slot = _Field('slot')
    orientation = _Field('orientation')
    port = _Field('port')
    link_info = _Field('link_info')
    fabric_info = _Field('fabric_info')


class SnapshotFields():
    id = _Field('id')
    name = _Field('name')
    description = _Field('description')
    size = _Field('size')
    vol_name = _Field('vol_name')
    pool_name = _Field('pool_name')
    vol_id = _Field('vol_id')
    snap_collection_name = _Field('snap_collection_name')
    snap_collection_id = _Field('snap_collection_id')
    online = _Field('online')
    writable = _Field('writable')
    offline_reason = _Field('offline_reason')
    expiry_time = _Field('expiry_time')
    expiry_after = _Field('expiry_after')
    origin_name = _Field('origin_name')
    is_replica = _Field('is_replica')
    is_unmanaged = _Field('is_unmanaged')
    is_manually_managed = _Field('is_manually_managed')
    replication_status = _Field('replication_status')
    access_control_records = _Field('access_control_records')
    serial_number = _Field('serial_number')
    target_name = _Field('target_name')
    creation_time = _Field('creation_time')
    last_modified = _Field('last_modified')
    schedule_name = _Field('schedule_name')
    schedule_id = _Field('schedule_id')
    app_uuid = _Field('app_uuid')

    @classmethod
    def metadata(cls, key):
        return _Field('metadata.' + key)

    new_data_valid = _Field('new_data_valid')
    new_data_compressed_bytes = _Field('new_data_compressed_bytes')
    new_data_uncompressed_bytes = _Field('new_data_uncompressed_bytes')
    agent_type = _Field('agent_type')
    vpd_t10 = _Field('vpd_t10')
    vpd_ieee0 = _Field('vpd_ieee0')
    vpd_ieee1 = _Field('vpd_ieee1')
    force = _Field('force')


class KeyManagerFields():
    id = _Field('id')
    name = _Field('name')
    description = _Field('description')
    hostname = _Field('hostname')
    port = _Field('port')
    protocol = _Field('protocol')
    username = _Field('username')
    password = _Field('password')
    active = _Field('active')
    status = _Field('status')
    vendor = _Field('vendor')


class NetworkInterfaceFields():
    id = _Field('id')
    array_name_or_serial = _Field('array_name_or_serial')
    partial_response_ok = _Field('partial_response_ok')
    array_id = _Field('array_id')
    controller_name = _Field('controller_name')
    controller_id = _Field('controller_id')
    name = _Field('name')
    mac = _Field('mac')
    is_present = _Field('is_present')
    link_speed = _Field('link_speed')
    link_status = _Field('link_status')
    mtu = _Field('mtu')
    port = _Field('port')
    slot = _Field('slot')
    max_link_speed = _Field('max_link_speed')
    nic_type = _Field('nic_type')
    ip_list = _Field('ip_list')



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class NetworkInterface(Resource):
    """Manage per array network interface configuration.

    # Parameters
    id                   : Identifier for the interface.
    array_name_or_serial : Name or serial of the array where the interface is hosted.
    partial_response_ok  : Indicate that it is ok to provide partially available response.
    array_id             : Identifier for the array.
    controller_name      : Name (A or B) of the controller where the interface is hosted.
    controller_id        : Identifier of the controller where the interface is hosted.
    name                 : Name of the interface.
    mac                  : MAC address of the interface.
    is_present           : Whether this interface is present on this controller.
    link_speed           : Speed of the link.
    link_status          : Status of the link.
    mtu                  : MTU on the link.
    port                 : Port number for this interface.
    slot                 : Slot number for this interface.
    max_link_speed       : Maximum speed of the link.
    nic_type             : Interface type.
    ip_list              : List of IP addresses assigned to this network interface.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class NetworkInterfaceList(Collection):
    resource = NetworkInterface
    resource_type = "network_interfaces"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#


class Resource:
    __slots__ = ['__id', '__attrs', '_collection', '_client']

    def __init__(self, id, attrs=None, client=None, collection=None):
        self.__id = id
        self.__attrs = {} if attrs is None else attrs
        self._collection = collection
        self._client = client

    @property
    def id(self):
        return self.__id

    @property
    def attrs(self):
        return self.__attrs

    def reload(self):
        self.__attrs = self._collection.get(self.__id).__attrs

    def update(self, **kwargs):
        resp = self._client.update_resource(self._collection.resource_type, self.__id, **kwargs)
        self._attrs = resp

    def delete(self, **kwargs):
        return self._client.delete_resource(self._collection.resource_type, self.__id)

    def __repr__(self):
        if 'name' in self.attrs:
            return f"<{self.__class__.__name__}(id={self.__id}, name={self.attrs['name']})>"
        else:
            return f"<{self.__class__.__name__}(id={self.__id})>"

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__id == other.__id

    def __hash__(self):
        return hash(f"{self.__class__.__name__}:{self.__id}")


class Collection:
    __slots__ = ['resource', 'resource_type', '_client']

    def __init__(self, client=None):
        self._client = client

    def get(self, id=None, **kwargs):
        if id is not None:
            obj = self._client.get_resource(self.resource_type, id)
            return self.resource(obj['id'], obj, client=self._client, collection=self)

        elif kwargs is not None:
            objs = self._client.list_resources(self.resource_type, detail=True, **kwargs)
            if len(objs) == 0:
                return None
            else:
                return self.resource(objs[0]['id'] if 'id' in objs[0] else None, objs[0], client=self._client, collection=self)

    def create(self, name, **kwargs):
        resp = self._client.create_resource(self.resource_type, name=name, **kwargs)
        return self.resource(resp['id'], resp, client=self._client, collection=self)

    def update(self, id, **kwargs):
        resp = self._client.update_resource(self.resource_type, id, **kwargs)
        return self.resource(resp['id'], resp, client=self._client, collection=self)

    def delete(self, id, job_timeout=None):
        return self._client.delete_resource(self.resource_type, id, job_timeout)

    def list(self, **kwargs):
        objs = self._client.list_resources(self.resource_type, **kwargs)
        return [self.resource(obj['id'] if 'id' in obj else None, obj, client=self._client, collection=self) for index, obj in enumerate(objs)]
